/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

var IconsInstance = function() {
    // These integers should be kept in sync with the IconType raw-values
    var ICON = 0;
    var APPLE = 1;
    var APPLE_PRECOMPOSED = 2;
    var SHORTCUT_ICON = 3;
    var BOOKMARK = 4;
    var GUESS = 5;
    
    var Favicons = {
    selectors: {
        "link[rel='icon']": ICON,
        "link[rel='apple-touch-icon']": APPLE,
        "link[rel='apple-touch-icon-precomposed']": APPLE_PRECOMPOSED,
        "link[rel='shortcut icon']": SHORTCUT_ICON,
        "link[rel='bookmark']": BOOKMARK
    },
    init: function() {
        
    },
    getAll: function() {
        var res = {};
        var foundIcons = false;
        
        for (selector in this.selectors) {
            var icons = document.querySelectorAll(selector);
            if (icons.length) {
                iconsFound = [];
                for (var i = 0; i < icons.length; i++) {
                    var href = icons[i].href;
                    var iconInfo = {"url": href};
                    var size = icons[i].getAttribute("sizes");
                    if (size) {
                        iconInfo["size"] = size;
                    }
                    iconsFound.push(iconInfo);
                    foundIcons = true;
                }
                res[this.selectors[selector]] = iconsFound;
            }
        }
        
        // If we didn't find anything in the page, look to see if a favicon.ico file exists for the domain
        if (!foundIcons) {
            var href = document.location.origin + "/favicon.ico";
            res[GUESS] = [{"url": href}];
        }
        return res;
    }
    }
    return Favicons;
}();

var FaviconsInstance = function() {
    // These integers should be kept in sync with the IconType raw-values
    var ICON = 0;
    var APPLE = 1;
    var APPLE_PRECOMPOSED = 2;
    var GUESS = 3;
    
    var Favicons = {
    selectors: { "link[rel~='icon']": ICON,
        "link[rel='apple-touch-icon']": APPLE,
        "link[rel='apple-touch-icon-precomposed']": APPLE_PRECOMPOSED
    },
    init: function() {
        
    },
    getAll: function() {
        var res = {}
        var foundIcons = false
        
        for (selector in this.selectors) {
            var icons = document.querySelectorAll(selector)
            if (icons.length) {
                res = {}
            }
            for (var i = 0; i < icons.length; i++) {
                var href = icons[i].href;
                res[href] = this.selectors[selector];
                foundIcons = true
            }
        }
        
        // If we didn't find anything in the page, look to see if a favicon.ico file exists for the domain
        if (!foundIcons) {
            var href = document.location.origin + "/favicon.ico";
            res[href] = GUESS;
        }
        
        return res;
    }
    }
    return Favicons;
}();

function scanDocumentModel() {
    var ICON = 0;
    var APPLE = 1;
    var APPLE_PRECOMPOSED = 2;
    var SHORTCUT_ICON = 3;
    var BOOKMARK = 4;
    var GUESS = 5;
    
    var info = {};
    
    // titel, key_current_title
    var title = document.title;
    
    info["title"] = title;
    info["key_current_title"] = title;
    
    // url, key_current_url
    var URL = window.location.href;
    info["url"] = URL;
    info["key_current_url"] = URL;
    
    // description
    var meta = document.querySelector("meta[name='description']");
    if (meta) {
        meta = meta.getAttribute("content");
    }
    info["description"] = meta;
    
    // Icons
    var allIcons = IconsInstance.getAll();
    
    var bookmarkIcons = allIcons[BOOKMARK];
    if (bookmarkIcons && bookmarkIcons[0]) {
        var firstBookmarkIcons = bookmarkIcons[0];
        if (firstBookmarkIcons) {
            info["image"] = firstBookmarkIcons["url"];
        }
    }
    
    var shortcutIcons = allIcons[SHORTCUT_ICON];
    if (shortcutIcons && shortcutIcons[0]) {
        var firstShortcutIcon = shortcutIcons[0];
        info["shortcut_icon"] = firstShortcutIcon["url"];
        info["image"] = firstShortcutIcon["url"];
    }
    
    var favIcons = allIcons[GUESS];
    if (favIcons && favIcons[0]) {
        var firstFavIcon = favIcons[0];
        info["fav_icon"] = firstFavIcon["url"];
    }
    
    var appleIconPrecomposed = allIcons[APPLE_PRECOMPOSED];
    if (appleIconPrecomposed) {
        info["apple_touch_icon_precomposed"] = appleIconPrecomposed;
        info["image"] = info["apple_touch_icon_precomposed"][0]["url"];
    }
    
    var appleIcon = allIcons[APPLE];
    if (appleIcon) {
        info["apple_touch_icon"] = appleIcon;
        info["image"] = info["apple_touch_icon"][0]["url"];
    }
    
    var icons = allIcons[ICON];
    if (icons && icons[0]) {
        var firstIcon = icons[0];
        info["icon"] = firstIcon["url"];
        info["image"] = info["icon"];
    }
    
    console.log(info);
    
    return info;
}