(function(global) {
  //'use strict';
  var ua = global.navigator.userAgent.toLowerCase(),
    doc = global.document,
    search = global.location.search,
    util;

  //IOS anti hijacking
  if (isIOS) {
    MUICordovaWebJSExport.addDNSOptimizeURL('news.web.sdp.101.com');
    MUICordovaWebJSExport.addDNSOptimizeURL('interaction.web.sdp.101.com');
    MUICordovaWebJSExport.addDNSOptimizeURL('aqapi.101.com');
  }

  //get search params
  var getUrlParams = (function() {
    var paramArr = search.match(/[=%\w]+/ig),
      results = {};

    paramArr.forEach(function(param) {
      param = param.split('=');
      var name = param[0];
      var value = param[1];
      results[name] = value;
    });

    return results;
  })();

  var ENV = getUrlParams.net;
  var LANG = getUrlParams.language;
  /^((zh)|(zh-CN))$/.test(LANG) && (LANG = 'zh');
  /^((id)|(in))$/.test(LANG) && (LANG = 'id');

  var UID = getUrlParams.uid;
  var i18n_index, fileName, serviceConfig, getNewsApi;

  //what is your platform
  var isAndroid = /android/.test(ua);
  var isIOS = /iphone|ipod|ipad|ios/.test(ua);
  var isSupportLocalStorage = global.localStorage ? true : false;
  var isCordovaSupport = false;
  var isCordovaReady = false;

  if (isAndroid) {
    serviceConfig = JSON.parse(jsConfigTools.getServiceConfig());
  } else if ((isIOS) &&　window.MUICordovaWebJSExport) {
    serviceConfig = MUICordovaWebJSExport.getServiceConfig();
  }
  if(serviceConfig && serviceConfig.news_url){
      getNewsApi = serviceConfig.news_url;
      var notEnd = /[^/]$/
      // 结尾补 "/" 符号
      notEnd.test(getNewsApi) && (getNewsApi += "/")
  } else if (ENV) {
    //ajax API according to ENV
    //Be short of interaction and content API
    getNewsApi = (function(net) {
      switch (net) {
        case 'test':
          return 'http://news.debug.web.nd/v0.2/';
        case 'dev':
          return 'http://news.dev.web.nd/v0.2/';
        case 'beta':
          return 'http://news.beta.web.sdp.101.com/v0.2/';
        case 'aws':
          return 'http://news.aws.101.com/v0.2/';
        case 'partyhome':
          return 'http://news.dyejia.cn/v0.2/';
        default:
          return 'http://news.web.sdp.101.com/v0.2/';
      }
    })(ENV);
  }

  //loading cordova js file
  if (isAndroid) {
    doc.write("<script src='./js/lib/cordova/cordova.js'><\/script>");
    isCordovaSupport = true;
  }
  if (isIOS) {
    doc.write("<script src='./js/lib/cordova/cordova_ios.js'><\/script>");
    isCordovaSupport = true;
  }

  if (isCordovaSupport) {
    doc.addEventListener('deviceready', function() {
      isCordovaReady = true;
      onEvent(EVENT_CORDOVA_READY);
    }, false);
  }

  //native toast
  window.NativeToast = function(msg) {
    if (isAndroid) {
      cordova.plugins.CommonServiceApi.nativeToastWithFinish(msg, false, "");
    }
    if (isIOS) {
      cordova.plugins.FPlugin.toast(msg);
    }
  };

  function Toast(message) {
    if (window.NativeToast) {
      if (isCordovaReady) {
        window.NativeToast(message);
      } else {
        document.addEventListener('deviceready', function() {
          window.NativeToast(message);
        }, false);
      }
    }
  }

  //receive data
  var REQUEST = {
    main: function(options, success, failure) {
      var data = options.data;
      var url = options.url;
      var method = options.method;

      if (method != 'GET') {
        data = JSON.stringify(data);
      }

      if (url.indexOf("://") < 1) {
        url = getNewsApi + url;
      }

      var params = [{
        'url': url
      }, data, {
        'method': method
      }];

      var str = new Date().getTime().toString(16) + Math.random().toString(16).substring(2);
      var scb = 's' + str;
      var fcb = 'f' + str;

      window[scb] = function(data) {
        return success(data);
      };
      window[fcb] = function(data) {
        var statusCode = data.code;
        var responseText = data.msg;
          //alert(JSON.stringify(data));
          //{"msg": "Network Error!","code":"timeout"}
          if (typeof statusCode === "string" && statusCode !== 'timeout') {
            if (/^\w+\/OUT_OF_SERVICE$/.test(statusCode)) {
              Toast(responseText);
            } else if (statusCode !== "WAF/GUEST_ACCESS_DENIED" && (statusCode !== "UC/AUTH_INVALID_TOKEN") && (statusCode !== 'UC/AUTH_INVALID_TIMESTAMP')) {
              Toast(i18n_index.statusCode);
            }
          } else {
            if (!statusCode || statusCode === 408 || statusCode === 504 || statusCode === 'timeout' || !responseText) {
              Toast(i18n_index.network_slow);
            }

            if (statusCode === 401 || statusCode === 403) {
              if (responseText.indexOf("时间戳") > 0) {
                Toast(i18n_index.your_phone_display_wrong_time, function() {
                  $.gotoLogin();
                });
              } else {
                $.gotoLogin();
              }
            }

            if (statusCode === 1200) {
              Toast(i18n_index.parameter_error);
            }
          }

        return failure(data);
      };

      if (isAndroid) {
        JsHttpRequest.execute(JSON.stringify(params), scb, fcb);
      }
      if (isIOS) {
        MUICordovaWebJSExport.execute(JSON.stringify(params), scb, fcb);
      }
    },
    get: function(options, success, failure) {
      options.method = 'GET';
      return this.main(options, success, failure);
    }
  };

  //category constructor function
  function Category(id) {
    this.id = id; // 分类ID
    this.desc = ""; // 描述
    this.name = ""; // 名称
    this.sort = 0; // 排序

    this.lastUpdateNewsTime = 0;
    this.newsRequested = false; // 是否 获取过新闻信息
    this.moreOlderNews = true;

    this.maxTs = -1; // 最大时间（最新）
    this.minTs = -1; // 最小时间（最旧）

    this.hasNews = false; // 是否有分类的新闻
    this.appendSize = 0;

    this.newsArray = [];
    this.thumbNews = [];
  }

  //detail work
  var mCategories = [];
  var categoriesKey = UID + '_key_store_categories';

  var TASK = {
    getCategories: function(callback) {
      REQUEST.get({
        url: '/categories',
        data: {
          size: 100
        }
      }, function(data) {
        var items = data.items;

        if (!data || items && items.length === 0) {
          callback(data, 0);
          return;
        }

        items.forEach(function(item) {
          TASK.insertOrUpdateCategory(item);
        });

        TASK.storeCategories(data);
        callback(data, true);
      }, function(data) {
        //dbcache缓存分类，断网的时候取缓存
        var categories = TASK.restoreCategoriesData();
        if (!data.code || data.code==='timeout') {
          callback(categories, 600);
          Toast(i18n_index.news_list_network_anomaly);
          return;
        }

        if (!categories || categories && categories.length === 0) {
          callback(categories, 0);
          return;
        }
        for (var j = 0, max = categories.length; j < max; j++) {
          var item = categories[j];
          TASK.insertOrUpdateCategory(item);
        }

        callback(categories, true);
      });
    },
    insertOrUpdateCategory: function(category) {
      var cid = category.id;
      if (!cid) return;

      var found = TASK.findCategory(cid);
      if (!found) {
        mCategories.push(TASK.buildCategory(category));
      } else {
        TASK.updateCategory(category, found);
      }
    },
    findCategory: function(categoryID) {
      return TASK.findItem(mCategories, categoryID);
    },
    findItem: function(dataContainer, id) {
      if (dataContainer.length === 0) return;
      dataContainer.forEach(function(item) {
        if (item.id === id) {
          return item;
        }
      });
    },
    buildCategory: function(data) {
      var category = new Category(data.id);
      TASK.updateCategory(data, category);
      return category;
    },
    updateCategory: function(src, dest) {
      if (!src || !dest) {
        return;
      }
      dest.desc = src.desc;
      dest.name = src.name;
      dest.sort = src.sort;
    },
    storeCategories: function(data) {
      if (isSupportLocalStorage) {
        if (isAndroid) {
          JsSaveSp.removeItem(categoriesKey);
          JsSaveSp.setItem(categoriesKey, JSON.stringify(data));
        }
        if (isIOS) {
          localStorage.removeItem(categoriesKey);
          localStorage.setItem(categoriesKey, JSON.stringify(data));
        }
      }
    },
    restoreCategoriesData: function() {
      if (isSupportLocalStorage) {
        var restore = TASK.restoreLocalJsonData(categoriesKey);

        if (!restore) {
          return [];
        }

        return restore;
      }
    },
    restoreLocalJsonData: function(key) {
      if (isSupportLocalStorage) {
        var data;
        if (isAndroid) {
          data = JsSaveSp.getItem(key);
        }
        if (isIOS) {
          data = localStorage.getItem(key);
        }

        if (!data) {
          return;
        }
        return JSON.parse(data);
      }
    }
  };

  function pushCategoriesData(data) {
    if (isAndroid) {
      cordova.plugins.CommonServiceApi.categories(data);
    } else if (isIOS) {
      cordova.plugins.FPlugin.initCategoryView(null, null, data);
      cordova.plugins.FPlugin.actionFinishLoad();
    }
  }

  //event register
  var EVENT_CORDOVA_READY = "event_cordova_ready";
  var mEventBus = [];

  function Event(id, callback) {
    this.id = id;
    this.callback = callback;
  }

  function insertOrUpdateEvent(event) {
    if (event.id === undefined) return false;

    if (mEventBus.length <= 0) {
      mEventBus.push(event);
      return;
    }

    var added = true;
    for (var i = 0, max = mEventBus.length; i < max; i++) {
      if (mEventBus[i].id == event.id) {
        mEventBus[i] = event;
        added = false;
        break;
      }
    }

    if (added) {
      mEventBus.push(event);
    }
  }

  function registerEventBus(id, callback) {
    insertOrUpdateEvent(new Event(id, callback));
  }

  function onEvent(event) {
    for (var i = 0; i < mEventBus.length; i++) {
      var item = mEventBus[i];
      if (item !== undefined) {
        item.callback(event);
      }
    }
  }

  function getCategory(data) {
    i18n_index = data;
    doc.title = data.news_center;
    TASK.getCategories(function(categories, statusCode) {
      if (statusCode === true && categories) {
        if (isCordovaSupport) {
          if (isCordovaReady) {
            pushCategoriesData([categories]);
          } else {
            registerEventBus("indexCtrl_pushCategoriesData", function(event) {
              if (EVENT_CORDOVA_READY === event) {
                pushCategoriesData([categories]);
              }
            });
          }
        }
      } else {
        $('#loadImg').hide();

        if (statusCode === 0) {
          $('#noCategory').show();
          $('#noCategory .app-text').text(i18n_index.news_list_no_category);
        }

        if (statusCode === 600) {
          $('#networkError').show();
          $('#networkError .app-text').text(i18n_index.network_error);
        }
      }
    });
  }

  $.getJSON('./languages/' + LANG + '.json', function(data) {
      getCategory(data);
    })
    .fail(function() {
      $.getJSON('./languages/en.json', function(data) {
        getCategory(data);
      });
    });


})(this);
