/**
 * 更新 或者 插入 对象（避免出现重复ID）
 * @param dataContainer
 * @param item
 * @returns {boolean}
 */
function insertOrUpdate(dataContainer, item) {
  if (item.id === undefined) return false;

  if (dataContainer.length <= 0) {
    dataContainer.push(item);
    return true;
  }

  var added = true;
  for (var i = 0; i < dataContainer.length; i++) {
    if (dataContainer[i].id == item.id) {
      dataContainer[i] = item;
      added = false;
      break;
    }
  }

  if (added) {
    dataContainer.push(item);
  }

  return added;
}

function insertArrayToArray(target, needToInsert) {
  if (needToInsert === undefined || needToInsert.length <= 0) return;
  for (var i = 0; i < needToInsert.length; i++) {
    insertToArray(target, needToInsert[i]);
  }
}

function insertToArray(array, item) {
  if (item === undefined) return;
  for (var i = 0; i < array.length; i++) {
    if (array[i] === item) return;
  }
  array.push(item);
}

/**
 * 通过ID 查找对象
 * @param dataContainer
 * @param id
 * @returns {*}
 */
function findItem(dataContainer, id) {
  if (dataContainer.length <= 0) return undefined;
  for (var i = 0; i < dataContainer.length; i++) {
    if (dataContainer[i].id == id) return dataContainer[i];
  }
  return undefined;
}

/*------------------------------------------------------------------------------*/


function getIonicListHeightPatch(id) {
  //return $("#ion-list-commont > div").outerHeight(true)
  var height = $("#" + id + " > div").outerHeight(true);
  log("getIonicListHeightPatch :: height-->" + height);
  return height;
}

/**
 * 判断 网络是否连接
 * @returns {boolean}
 */
function isNetworkConnected() {
  return navigator.onLine;
}

//function removeScript(s) {
//    return s.replace(/<script.*?>.*?<\/script>/ig, '').replace(/</ig,'&lt;').replace(/>/ig, '&gt;');
//}

function replaceAngleBracket(s) {
  return s.replace(/&lt;/ig, '<').replace(/&gt;/ig, '>');
}


/**
 * 获取当前时间
 * @returns {number}
 */
function getNowUnixTime() {
  return Date.parse(new Date());
}

/*
 * 日期格式化
 */
Date.prototype.format = function(format) {
  var o = {
    "M+": this.getMonth() + 1, //month
    "d+": this.getDate(), //day
    "h+": this.getHours(), //hour
    "m+": this.getMinutes(), //minute
    "s+": this.getSeconds(), //second
    "q+": Math.floor((this.getMonth() + 3) / 3), //quarter
    "S": this.getMilliseconds() //millisecond
  };

  if (/(y+)/.test(format)) {
    format = format.replace(RegExp.$1, (this.getFullYear() + "").substr(4 -
      RegExp.$1.length));
  }

  for (var k in o) {
    if (new RegExp("(" + k + ")").test(format)) {
      format = format.replace(RegExp.$1, RegExp.$1.length == 1 ? o[k] : ("00" + o[k]).substr(("" + o[k]).length));
    }
  }
  return format;
};


function convertISO8601toDate(dtstr) {
  if (!dtstr) {
    return dtstr;
  }

  //没有Z，有时区，+0800或其他时区
  var timeZone = 0;
  if(dtstr.indexOf('Z')===-1){
    timeZone = dtstr.slice(-5, -2);
  }

  // replace anything but numbers by spaces
  dtstr = dtstr.replace(/\D/g, " ");

  // trim any hanging white space
  dtstr = dtstr.replace(/\s+$/, "");

  // split on space
  var dtcomps = dtstr.split(" ");

  // not all ISO 8601 dates can convert, as is
  // unless month and date specified, invalid
  if (dtcomps.length < 3) return "invalid date";
  // if time not provided, set to zero
  if (dtcomps.length < 4) {
    dtcomps[3] = 0;
    dtcomps[4] = 0;
    dtcomps[5] = 0;
  }

  // modify month between 1 based ISO 8601 and zero based Date
  dtcomps[1]--;

  var convdt = new
  Date(Date.UTC(dtcomps[0], dtcomps[1], dtcomps[2], dtcomps[3]-timeZone, dtcomps[4], dtcomps[5]));

  return convdt.toUTCString();
}

/**
 * 转换新闻正文中的 图片内容
 * @param content
 * @returns {*}
 */
function parse_news_image(content) {
  var urls;
  var imgWidth = document.documentElement.clientWidth - 20;

  //解析img标签的data-src属性值
  var PATTERN_DENTRYID = /([0-9a-f]{8}(?:\-[0-9a-f]{4}){3}\-[0-9a-f]{12})/g;

  if (!content) {
    return;
  }

  var ids = content.match(PATTERN_DENTRYID);

  if (!ids) {
    return content;
  }

  var dentryids = ids.map(function(item) {
    return {
      'dentryId': item,
      'size': 960
    };
  });

  urls = getImgUrls(JSON.stringify(dentryids));

  urls = JSON.parse(urls);

  urls.forEach(function(item, i) {
    var dentryid = item.dentryId;
    var url = item.url;
    var replaceText = ' onclick="normalGoBigPicView(\''+ url + '\')" src="' + url + '"';

    content = content.replace(new RegExp('data-src=[\'\"]' + dentryid + '[\'\"]', 'img'), replaceText);
  });

  //如果是图片链接，点开不打开大图
  content = content.replace(/<a href=[^<]*<img[^<]*<\/a>/img, function(match){
    return match.replace(/ onclick=("|')normalGoBigPicView\([^\)]*\)("|')/, '');
  });

  return content;
}


function getContentServiceHost() {
  var csAPI = localStorage.getItem(KEY_STORE_CS_API);
  return csAPI + 'download?dentryId='
}


function convertImageURLWithSize(url, size) {
  //绝对路径直接返回
  if ((/^http:\/\//ig.exec(url)) ||
    (/^https:\/\//ig.exec(url)) ||
    (/^file:\/\//ig.exec(url)) ||
    (/^ftp:\/\//ig.exec(url))) {
    return url;
  }

  var fitSize;
  if (size <= 80) {
    fitSize = 80;
  } else if (size > 80 && size <= 120) {
    fitSize = 120;
  } else if (size > 120 && size <= 160) {
    fitSize = 160;
  } else if (size > 160 && size <= 240) {
    fitSize = 240;
  } else if (size > 240 && size <= 320) {
    fitSize = 320;
  } else if (size > 320 && size <= 480) {
    fitSize = 480;
  } else if (size > 480 && size <= 640) {
    fitSize = 640;
  } else if (size > 640) {
    fitSize = 960;
  }
  return getContentServiceHost() + url + (fitSize !== undefined ? "&size=" + fitSize : "");
}


String.prototype.replaceAll = function(s1, s2) {
  return this.replace(new RegExp(s1, "gm"), s2);
};

/**
 * 替换 字符串中的http链接
 * @param text
 * @returns {*}
 */
function replaceURLWithHTMLLinks(text) {

  var exp = /(\b(https?|ftp|file):\/\/[-A-Z0-9+&@#\/%?=~_|!:,.;]*[-A-Z0-9+&@#\/%=~_|])/ig;
  return text.replace(exp, "<a href='$1' target='_blank'>$1</a>");
}

/*------------------------------------------------------------------------------*/

/**
 * 跳转 首页
 */
function toPageMain() {
  toPageOrCordova("新闻中心", "newsList.html", true);
}

/**
 * 跳转 新闻详情 - 图文
 */
function toNewsDetailNormal(categoryID, newsID) {
  timeLog(2, "Start");

  toPageOrCordova("detail", "newsDetailNormal.html?" +
    "newsid=" + newsID +
    "&categoryid=" + categoryID, false);
}

/**
 * 跳转 新闻详情 图集 并携带翻页位置
 */
function toNewsDetailGallery(categoryID, newsID, slide) {
  toPageOrCordova("detail", "newsDetailGallery.html?" +
    "categoryid=" + categoryID +
    "&newsid=" + newsID +
    "&slide=" + slide, false);
}

/**
 * 从专题页打开新闻详情
 * @param categoryID
 * @param topicID
 * @param newsID
 */
function toTopicNewsDetailNormal(categoryID, topicID, newsID) {
  toPageOrCordova("detail", "newsDetailNormal.html?" +
    "categoryid=" + categoryID +
    "&newsid=" + newsID +
    "&topicid=" + topicID, false);
}

/**
 * 跳转 专题
 */
function toPageTopicDetail(categoryID, topicID) {
  toPageOrCordova("topic", "topicDetail.html?"+
    "categoryid=" + categoryID +
    "&topicid=" + topicID, true);
}

/**
 * 跳转 图片浏览
 */
function toPagePhoto() {
  toPageOrCordova("photo", "photoView.html");
}

/**
 * 跳转 评论
 */
function toPageComments(newsID) {
  toPageOrCordova("comment", "comments.html?" + "newsid=" + newsID, true);
}

/**
 * 跳转 点赞详情页
 */
function toNewsPraisers(newsID) {
  toPageOrCordova("praisers", "newsPraisers.html?" + "newsid=" + newsID, true);
}


/**
 * 跳转 cordova 原生页面新开
 * @param params 跳转参数
 * @param url 地址
 * @param pullRefresh 是否需要刷新
 */
function toPageOrCordova(params, url, pullRefresh) {
  var dest;
  if (mCordovaSupport) {
    if (mPlatformAndroid) {
      dest = mOfflineMode ? "file:///android_asset/webapp/app/mobile/" + url : "http://" + location.host + "/app/mobile/" + url;
      cordova.plugins.CommonServiceApi.pageJump("action_skipTo", params, dest);
    } else if (mPlatformIOS) {
      dest = mOfflineMode ? "app/mobile/" + url : "http://" + location.host + "/app/mobile/" + url;
      if (pullRefresh) {
        cordova.plugins.FPlugin.actionSkipTo(null, null, params, dest);
      } else {
        cordova.plugins.FPlugin.actionSkipToWidthoutPullRefresh(null, null, params, dest);
      }
    }
  } else {
    location.href = url;
  }
}


/**
 * 返回上一页
 */
function goBack() {
  history.back();
}

/**
 * 查看大图
 */
function goBigPicView(imgs, index) {
  if (mCordovaSupport) {
    cordova.plugins.DeviceApi.picView(imgs, index);
  }
}

/**
 * 图文页面跳转到详情
 */
function normalGoBigPicView(url) {
  if (url) {
    goBigPicView([url], 0);
  }
}

/**
 * 从images数组中，取出url数组
 * @param images
 */
function getUrlsFromImages(images) {
  var size = Math.max(document.documentElement.clientWidth, document.documentElement.clientHeight);

  var imgs = [];
  for (var i = 0, max = images.length; i < max; i++) {
    imgs[i] = convertImageURLWithSize(images[i].url, size);
  }

  return imgs;
}


var settingData;

/**
 * 单张图片url获取
 * @param dentryId
 * @param size
 * @return url
 */
function getImgUrl(dentryId, size) {
  if (mPlatformAndroid) {
    return JsHttpRequest.getImgUrl(dentryId, +size);
  }else if (mPlatformIOS) {
    return MUICordovaWebJSExport.getPhotoURLString(dentryId, +size);
  }
}

/**
 * 多张图片url获取
 * @param String options
 * @return String urls
 */
function getImgUrls(options) {
  var urls;

  if (mPlatformAndroid) {
    urls = JsHttpRequest.getImgUrls(options);
  }else if (mPlatformIOS) {
    urls = MUICordovaWebJSExport.getPhotoURLStringArray(options);
  }

  return urls;
}


