// 新闻评论详情页 控制器

angular.module('news.controller.comment', ['news.service.cordova', 'i18n_module'])
  .controller('CommentsPageCtrl', ['$rootScope', '$scope', '$timeout', '$location', '$sce', '$translate', 'cordovaService', '$filter',
    function($rootScope, $scope, $timeout, $location, $sce, $translate, $cordovaService, $filter) {

      $scope.inited = false;
      $scope.mNetworkErr = false;
      $scope.hasMask = false;
      var newsID = $scope.newsID = $.getUrlParam('newsid');
      var categoryID = getCurrentDetailNewsCategoryID(),
        analyticsParam, userinfo, isLoginEvent = false;

      /**
       * 是否正在提交（防止弱网时重复提交多次）
       * @type {boolean}
       */
      $scope.isPostingComment = false;
      $scope.mScrollerHeight = document.documentElement.clientHeight - 50;

      registerEventBus("CommentsPageCtrl_element_refresh", function(event) {
        if (EVENT_ELEMENT_REFRESH === event) {
          $scope.$digest();
        }
      });

      $scope.toast = function(msg) {
        $cordovaService.toast(msg);
      };

      $scope.init = function() {

        if (mCordovaSupport) {
          if (mCordovaReady) {
            $cordovaService.addPullRefreshForIos();
            _addCommentBarForIos();
          } else {
            registerEventBus("TopicPageCtrl_add_pull_refresh", function(event) {
              if (EVENT_CORDOVA_READY === event) {
                $cordovaService.addPullRefreshForIos();
                _addCommentBarForIos();
              }
            });
          }
        }

        if ($scope.newsID !== undefined) {
          $scope.getNewsPraisersLatest();
        }

        $scope.$digest();
      };

      $scope.showCommentBarIfAllow = function() {
        if (!mCordovaSupport) return true;
        return !mPlatformIOS;
      };

      var _addCommentBarForIos = function() {
        $cordovaService.addBottomInputForIos(140, 'window.postCommentFromOutside', "");
      };

      var _addFollowComment = function(comment) {
        if (mPlatformAndroid) {
          $cordovaService.followCommentForAndroid(comment);
        } else if (mPlatformIOS) {
          $cordovaService.addBottomInputForIos(140, 'window.postCommentFromOutside', comment);
        }
      };


      $scope.convertAsTrust = function(str) {
        return $sce.trustAsHtml(str);
      };

      $scope.hasComments = function() {
        return hasComments();
      };
      $scope.hasPraisers = function() {
        return hasPraisers();
      };

      /**
       * 获取 评论用户的名称
       * @param commentID
       * @returns {*}
       */
      $scope.getCommentUserName = function(comment) {
        return getCommentUserName(comment);
      };

      /**
       * 获取 上次数据请求的时间
       * @returns {*}
       */
      $scope.getLastRequestTime = function() {
        var time = getCommentRequestTime();
        if (time === undefined) {
          time = $filter('translate')('no_more_items');
        }
        return time;
      };

      /**
       * 获取评论回调
       */
      var requestCommentCallback = function(statusCode) {
        $scope.inited = true;
        $scope.mNetworkErr = statusCode === FAILURE;
        $scope.comments = mCurrentComments;

        $cordovaService.donePulldown();

        if (mCordovaReady) {
          $cordovaService.allowURLJumpforIos();
        } else {
          registerEventBus("TopicPageCtrl_allow_url_jump", function(event) {
            if (EVENT_CORDOVA_READY === event) {
              $cordovaService.allowURLJumpforIos();
            }
          });
        }

        if (!isLoginEvent) {
          userinfo = getLocaleUserInfo();

          if (userinfo) {
            var ipArea = getIpArea(userinfo);

            analyticsParam = {
              'categoryId': categoryID,
              'newsId': newsID,
              'province': ipArea[0],
              'city': ipArea[1]
            };
          }

          if (mPlatformAndroid) {
            analyticsParam = JSON.stringify(analyticsParam);
          }
        }


        $scope.$apply();

        angular.element('#ion-list-comment').show();
      };

      /**
       * 获取较早评论回调
       */
      var requestOlderCommentCallback = function(statusCode) {
        $scope.inited = true;
        $scope.comments = mCurrentComments;

        $scope.$apply();

        if (!hasOlderComment()) {
          $scope.toast($filter('translate')('comment_content_pull_bottom'));
        }

        $cordovaService.donePullup();
      };

      $scope.toNewsPraisers = function() {
        toNewsPraisers($scope.newsID);
      };
      /**
       * 更新 互动评论 计数器
       */
      $scope.updateInteractionsCounter = function() {
        getNewsInteractionsCounter(mCurrentDetailNewsID, function() {});
      };

      /**
       * 获取 最近评论
       */
      $scope.getCommentLatest = function() {
        $scope.mNetworkErr = false;

        getNewsCommentsLatest($scope.newsID, DEFAULT_REQUEST_SIZE, function(statusCode) {
          requestCommentCallback(statusCode);
        });
      };

      /**
       * 获取 用户头像
       */
      $scope.getUserAvatar = function(userID) {

        if (userID === undefined) {
          return './images/default_avatar.png';
        }
        if (mPlatformAndroid) {
          return jscallusericon.getUserIconUrl(userID, DEFAULT_AVATAR_SIZE);
        } else if (mPlatformIOS) {
          return MUICordovaWebJSExport.getUserAvatarURLString(userID, DEFAULT_AVATAR_SIZE);
        }

      };

      /**
       * 获取 较早的评论
       */
      $scope.getCommentOlder = function() {
        getNewsCommentsOlder($scope.newsID, DEFAULT_REQUEST_SIZE, function(statusCode) {
          requestOlderCommentCallback(statusCode);
        });
      };

      /**
       * 判断 当前是否还有更早的评论 需要获取
       * @returns {*}
       */
      $scope.hasOlderComment = function() {
        return hasOlderComment();
      };

      $scope.comment = {
        content: ""
      };

      /**
       * 获取过滤后的评论信息（防止JS注入）
       * @param str
       * @returns {*}
       */
      $scope.getFilteredStr = function(str) {
        var replacedStr = str.replaceAll('\n', '<br>'); // 替换 换行
        replacedStr = replacedStr.replaceAll(' ', '&nbsp;'); // 替换 空格
        replacedStr = replaceURLWithHTMLLinks(replacedStr);
        return replacedStr;
      };

      /**
       * 回复 评论
       * @param comment
       */
      $scope.followComment = function(comment) {
        var userInfo = findUserInfo(comment.ower_uid);
        if (userInfo !== null) {
          $scope.comment.content = $filter('translate')('comment_reply') + (userInfo.name || userInfo.id) + ":";

          _addFollowComment($scope.comment.content);
        }
      };

      /**
       * 举报 评论
       * @param comment
       */
      $scope.report = function(comment, event) {
        $scope.hasMask = true;
        event.stopPropagation();
        report(comment);
      };

      /**
       * 取消 举报
       */
      $scope.cancelReport = function() {
        $scope.hasMask = false;
        cancelReport();
      };

      /**
       * 确认举报
       */
      $scope.confirmReport = function(type, text) {
        $scope.hasMask = false;
        confirmReport(type, text);
      };

      /**
       * 删除 评论
       * @param comment
       */
      $scope.deleteCmt = function(comment, event) {
        $scope.hasMask = true;
        event.stopPropagation();
        deleteCmt(comment);
      };

      /**
       * 取消 删除
       */
      $scope.cancelDelete = function() {
        $scope.hasMask = false;
        cancelDelete();
      };

      /**
       * 确认删除
       */
      $scope.confirmDelete = function() {
        $scope.hasMask = false;
        confirmDelete(function() {
          $cordovaService.refreshInteraction();
        });
      };

      /**
       * 判断当前用户是不是评论者本人
       */
      $scope.isCommentOwner = function(comment) {
        var uid = getCurrentUid();

        return comment.ower_uid === +uid;
      };

      /**
       * 获取点赞用户详情
       */
      $scope.getNewsPraisersLatest = function() {
        getNewsPraisersLatest($scope.newsID, 20, function(statusCode) {
          $scope.newsPraisers = getCurrentPraisers();
          $scope.getCommentLatest();
        });
      };

      $scope.getNewsPraiserName = function(praiser) {
        return getPraiserName(praiser);
      };

      /*****************************************************************/

      window.translate = function(msg) {
        return $filter('translate')(msg);
      };

      window.startPullDownToRefresh = function() {
        var time = $filter('translate')('refresh_time') + $scope.getLastRequestTime();
        if (mPlatformAndroid) {
          webView.setRefreshTime(time);
        } else if (mPlatformIOS) {
          $cordovaService.setRefreshTimeForIos(time);
        }

      };

      window.onPullDownToRefresh = function() {
        if ($scope.hasMask) {
          $cordovaService.donePulldown();
          return;
        }

        $scope.getNewsPraisersLatest();
      };

      window.onPullUpToRefresh = function() {
        if ($scope.hasMask) {
          $cordovaService.donePullup();
          return;
        }
        $scope.getCommentOlder();
      };

      /**
       * 发送评论
       * @param comment
       */
      window.postCommentFromOutside = function(comment) {
        $scope.comment.content = comment;

        if (comment.length <= 0) {
          $scope.toast($filter('translate')('comment_content_not_null'));
        } else if (comment.length > 140) {
          $scope.toast($filter('translate')('comment_content_not_long'));
        } else if ($scope.isPostingComment) {
          $scope.toast($filter('translate')('comment_content_not_again'));
        } else {
          $scope.isPostingComment = true;
          postComment($scope.newsID, comment, function(statusCode) {
            $scope.isPostingComment = false;
            if (SUCCESS === statusCode) {
              if (mPlatformAndroid) {
                CommentService.handleSendCommentResult(true);
                $cordovaService.refreshInteraction();
              } else if (mPlatformIOS) {
                MUICordovaWebJSExport.handleSendCommentResult(1);
              }

              $scope.toast($filter('translate')('comment_content_success'));
              $scope.getCommentLatest(); // 获取最新的评论（主要是包括自己的）
              $scope.updateInteractionsCounter(); // 更新一下互动计数器

              if (userinfo) {
                analyticsEvent('social_news_comment', analyticsParam);
              }
            }
          });
        }
      };

      window.logInEven = function(uid) {
        if (uid) {

          var cpcSuccess = 'getCurrentUserSuccess';
          var cpcFail = 'getCurrentUserFail';

          //获取用户信息的成功回调
          window[cpcSuccess] = function(data) {
            userinfo = JSON.stringify(data);

            if (userinfo) {
              var ipArea = getIpArea(userinfo);

              analyticsParam = {
                'categoryId': categoryID,
                'newsId': newsID,
                'province': ipArea[0],
                'city': ipArea[1]
              };
            }

            if (mPlatformAndroid) {
              analyticsParam = JSON.stringify(analyticsParam);
            }
          };
          //获取用户信息的失败回调
          window[cpcFail] = function(data) {
            console.log("get user info error");
          };

          if (mPlatformAndroid) {
            JsHttpRequest.getCurrentUser(cpcSuccess, cpcFail);
          }
          if (mPlatformIOS) {
            MUICordovaWebJSExport.getUserInfo(cpcSuccess, cpcFail);
          }

          isLoginEvent = true;

          window.onPullDownToRefresh();
        }
      };
    }
  ]);
