// 图文新闻详情页 控制器
angular.module('news.controller.news_detail', ['ionic', 'i18n_module', 'news.service.cordova'])
  .controller('NewsDetailPageCtrl', ['$rootScope', '$scope', '$ionicLoading', '$timeout', '$ionicSlideBoxDelegate', '$ionicScrollDelegate', '$location', 'cordovaService', '$translate', '$filter',
    function($rootScope, $scope, $ionicLoading, $timeout, $ionicSlideBoxDelegate, $ionicScrollDelegate, $location, $cordovaService, $translate, $filter) {
      $scope.mScrollerHeight = document.documentElement.clientHeight - 35;
      $scope.mGalleryImgWidth = document.documentElement.clientWidth;
      $scope.mGalleryImgHeight = document.documentElement.clientHeight - 45;
      $scope.inited = false;
      $scope.mNetworkErr = false;
      $scope.initSlidePosition = $.getUrlParam('slide');
      $scope.slidePosition = ($scope.initSlidePosition !== undefined) ? $scope.initSlidePosition : 0;

      var newsID = $.getUrlParam('newsid');
      var categoryID = $.getUrlParam('categoryid');
      var topicID = $.getUrlParam('topicid'),
        analyticsParam, userinfo;

      /**
       * 滚屏事件
       * @param index
       */
      $scope.slideChanged = function(index) {
        $scope.slidePosition = index;
      };

      $scope.showIfAllow = function(index) {
        return Math.abs($scope.slidePosition - index) < 2;
      };

      /**
       * 获取SlideBox初始化索引
       * @returns {number}
       */
      $scope.getInitSlidePosition = function() {
        return $scope.initSlidePosition;
      };

      $scope.toast = function(msg) {
        $cordovaService.toast(msg);
      };

      $scope.init = function() {
        $scope.inited = false;
        $scope.mNetworkErr = false;
        $scope.serviceClosed = false;
        $scope.serviceStoped = false;
        $scope.layoutClass = 'app-background';

        setCurrentDetailNewsID(newsID);
        if (categoryID !== undefined) {
          setCurrentDetailNewsCategoryID(categoryID);
        } else if (topicID !== undefined) {
          setCurrentTopicID(topicID);
        }

        if ($location.search().slide !== undefined) {
          // 设置SlideBox初始化索引
          $scope.initSlidePosition = $location.search().slide;
        }

        if (newsID !== undefined) {
          getNewsDetail(newsID, function(statusCode) {
            if (SUCCESS === statusCode) {
              userinfo = getLocaleUserInfo();

              if (userinfo) {
                var ipArea = getIpArea(userinfo);

                analyticsParam = {
                  'categoryId': categoryID,
                  'newsId': newsID,
                  'province': ipArea[0],
                  'city': ipArea[1]
                };
              }

              if (mPlatformAndroid) {
                analyticsParam = JSON.stringify(analyticsParam);
              }

              // 当前 新闻详情的新闻对象
              $scope.mCurrNews = mCurrentDetailNews;
              $scope.publishTime = new Date($scope.mCurrNews.publish_time).format(DEFAULT_TIME_FORMAT);
              $scope.getPraiseNumber();

              if (userinfo) {
                analyticsEvent('social_news_addhits', analyticsParam);
              }
            }

            if (DELETED === statusCode) {

              if (mSupportLocalStorage) {
                var restore = restoreLocalJsonData(DELETE_MARK);
                if (restore === undefined) {
                  restore = [];
                }
                restore.push(new News(newsID));
                localStorage.setItem(DELETE_MARK, JSON.stringify(restore));
              }

              var msg = $filter('translate')("news_detail_gallery_js_already_delete");
              if (mCordovaReady) {
                _toastAndFinish(msg);
              } else {
                registerEventBus("NewsDetailPageCtrl_toastAndFinish", function(event) {
                  if (EVENT_CORDOVA_READY === event) {
                    _toastAndFinish(msg);
                  }
                });
              }

            }

            $scope.inited = true;
            $scope.mNetworkErr = statusCode === FAILURE;
            $scope.serviceClosed = statusCode === 404;
            $scope.serviceStoped = statusCode === 402;
            $scope.$digest();
            $timeout(function() {
              $ionicSlideBoxDelegate.update();
            }, 50);

          });
        }

      };

      var _toastAndFinish = function(msg) {
        if (mPlatformAndroid) {
          $cordovaService.toastAndFinish(msg);
        } else if (mPlatformIOS) {
          $cordovaService.toast(msg);
          //FastToast( msg );
          $timeout(function() {
            $cordovaService.actionFinish();
          }, 1000);
        }
      };

      var _dispatchNetworkErrTag = function() {
        var tag = angular.element("#tag_err");
        if ($scope.inited && $scope.mNetworkErr) {
          tag.show();
        } else {
          tag.hide();
        }
      };

      $scope.descToHtml = function(index, desc) {
        var str = '<div class="gallery-desc-index"><span class="gallery-index">' + index + '</span><span class="gallery-count">/' + $scope.mCurrNews.images.length + '</span></div>';
        if (desc) {
          str = str + '<div class="gallery-desc-desc">'+desc+'</div>';
        }
        return str;
      };

      $scope.layoutClass = 'app-background';

      /**
       * 获取 互动评论 回调
       * @param statusCode
       */
      var interactionsCounterCallback = function(statusCode) {
        var newsID = $scope.mCurrNews.id;
        var news = findCategoryNews(newsID);
        if (news !== undefined) {
          $scope.mCurrNews.praiseNum = news.praiseNum;
          $scope.mCurrNews.commentsNum = news.commentsNum;
          $scope.mCurrNews.isPraise = news.isPraise;
        }
        $scope.$digest();
        praiseCount = 0;
      };

      /**
       * 根据 当前点赞状态 获取当前点赞图标
       * @param news
       * @returns {string}
       */
      $scope.getPraiseIcon = function(news) {
        if (mCurrentDetailNews.isPraise) {
          return './images/like_click_32.png';
        } else {
          return './images/like_32.png';
        }
      };

      /**
       * 判断 是否屏蔽评论和赞
       * @param news
       * @returns string
       */
      $scope.isShowComments = function () {
        return isShowComments();
      };

      $scope.isShowShare = function () {
        return isShowShare();
      };

      /**
       * 获取点赞数目
       */
      $scope.getPraiseNumber = function() {
        getNewsInteractionsCounter(getCurrentDetailNewsID(), interactionsCounterCallback);
      };

      /**
       * 触发 点赞事件
       * @param newsID
       */
      var praiseCount = 0;
      $scope.invokePraiseNews = function(newsID) {
        if (praiseCount === 1) {
          return;
        }

        praiseCount += 1;
        if (mCurrentDetailNews.isPraise) {
          // 新闻已经被点赞过，取消点赞
          cancelPraiseNews(newsID, function(statusCode) {
            if (SUCCESS === statusCode) {
              getNewsInteractionsCounter(newsID, interactionsCounterCallback);
              $cordovaService.refreshInteraction();
            }else{
              praiseCount = 0;
            }
          });
        } else {
          // 新闻没有被点赞过，可以点赞
          praiseNews(newsID, function(statusCode) {
            if (SUCCESS === statusCode) {
              $scope.toast($filter('translate')("news_detail_gallery_js_verygood"));
              getNewsInteractionsCounter(newsID, interactionsCounterCallback);
              $cordovaService.refreshInteraction();

              if (userinfo) {
                analyticsEvent('social_news_praise', analyticsParam);
              }
            }else{
              praiseCount = 0;
            }
          });
        }
      };

      $scope.toNewsComments = function() {
        toPageComments(getCurrentDetailNewsID());
      };

      /**
       * 分享功能
       */
      $scope.share = function(news) {
        /* 已 summary 为摘要值
        var content = news.content == null ? '' : news.content;
        content = ['<div>',content,'</div>'].join('');//包裹在 div 容器里，方便后面去纯文本（踢除HTML标签）
        content =  $(content).text().substring(0,300).substring(0,300);
        */
        var summary = news.summary == null ? '' : news.summary;
        if (mPlatformAndroid) {
          ShareBox.open(categoryID, news.id, news.title, summary, news.thumbnail);
        } else if (mPlatformIOS) {
          MUICordovaWebJSExport.socialSharePresentSheetView(categoryID, news.id, news.title, summary, news.thumbnail);
        }
      };

      $scope.goBigPicView = function(index) {
        var imgs = getUrlsFromImages($scope.mCurrNews.images);
        goBigPicView(imgs, index);
      };

      /****************************************************************************/
      window.translate = function(msg) {
        return $filter('translate')(msg);
      };

      window.onWebviewWillUpdate = function() {
        if ($scope.mCurrNews !== undefined) {
          getNewsInteractionsCounter($scope.mCurrNews.id, interactionsCounterCallback);
        }
      };

      window.logInEven = function(uid) {
        if (uid) {
          var ndpcSuccess = 'getCurrentUserSuccess';
          var ndpcFail = 'getCurrentUserFail';

          //获取用户信息的成功回调
          window[ndpcSuccess] = function(data) {
            userinfo = JSON.stringify(data);

            if (userinfo) {
              var ipArea = getIpArea(userinfo);

              analyticsParam = {
                'categoryId': categoryID,
                'newsId': newsID,
                'province': ipArea[0],
                'city': ipArea[1]
              };
            }

            if (mPlatformAndroid) {
              analyticsParam = JSON.stringify(analyticsParam);
            }
          };
          //获取用户信息的失败回调
          window[ndpcFail] = function(data) {
            console.log("get user info error");
          };

          if (mPlatformAndroid) {
            $cordovaService.refreshInteraction();
            JsHttpRequest.getCurrentUser(ndpcSuccess, ndpcFail);
          }
          if (mPlatformIOS) {
            MUICordovaWebJSExport.getUserInfo(ndpcSuccess, ndpcFail);
          }
        }
      };

    }
  ]);
