// 图文新闻详情页 控制器
angular.module('news.controller.news_detail', ['i18n_module', 'news.service.cordova'])
  .controller('NewsDetailPageCtrl', ['$rootScope', '$scope', '$timeout', '$location', '$sce', '$translate', '$filter', 'cordovaService',
    function($rootScope, $scope, $timeout, $location, $sce, $translate, $filter, $cordovaService) {

      $scope.mScrollerHeight = document.documentElement.clientHeight - 50;
      $scope.mGalleryImgWidth = document.documentElement.clientWidth;
      $scope.mGalleryImgHeight = document.documentElement.clientHeight - 55;
      $scope.inited = false;
      $scope.mNetworkErr = false;

      var newsID = $.getUrlParam('newsid');
      var categoryID = $.getUrlParam('categoryid');
      var topicID = $.getUrlParam('topicid'),
        analyticsParam, userinfo;

      registerEventBus("NewsDetailPageCtrl_element_refresh", function(event) {
        if (EVENT_ELEMENT_REFRESH === event) {
          $scope.$digest();
        }
      });

      $scope.toast = function(msg) {
        $cordovaService.toast(msg);
      };

      $scope.init = function() {
        $scope.inited = false;
        $scope.mNetworkErr = false;
        $scope.serviceClosed = false;
        $scope.serviceStoped = false;
        $scope.layoutClass = 'app-background';

        setCurrentDetailNewsID(newsID);
        if (categoryID !== undefined) {
          setCurrentDetailNewsCategoryID(categoryID);
        } else if (topicID !== undefined) {
          setCurrentTopicID(topicID);
        }

        if (newsID !== undefined) {
          getNewsDetail(newsID, function(statusCode) {
            if (SUCCESS === statusCode) {
              userinfo = getLocaleUserInfo();

              if (userinfo) {
                var ipArea = getIpArea(userinfo);

                analyticsParam = {
                  'categoryId': categoryID,
                  'newsId': newsID,
                  'province': ipArea[0],
                  'city': ipArea[1]
                };
              }

              if (mPlatformAndroid) {
                analyticsParam = JSON.stringify(analyticsParam);
              }

              // 当前 新闻详情的新闻对象
              $scope.mCurrNews = mCurrentDetailNews;
              $scope.publishTime = new Date($scope.mCurrNews.publish_time).format(DEFAULT_TIME_FORMAT);
              $scope.getPraiseNumber();

              if (userinfo) {
                analyticsEvent('social_news_addhits', analyticsParam);
              }
            }

            if (DELETED === statusCode) {
              var msg = $filter('translate')("news_detail_normal_js_already_delete");
              if (mCordovaReady) {
                _toastAndFinish(msg);
              } else {
                registerEventBus("NewsDetailPageCtrl_toastAndFinish", function(event) {
                  if (EVENT_CORDOVA_READY === event) {
                    _toastAndFinish(msg);
                  }
                });
              }

            }

            $scope.inited = true;
            $scope.mNetworkErr = statusCode === FAILURE;
            $scope.serviceClosed = statusCode === 404;
            $scope.serviceStoped = statusCode === 402;
            $scope.$digest();

          });
        }

      };

      var _toastAndFinish = function(msg) {
        if (mPlatformAndroid) {
          $cordovaService.toastAndFinish(msg);
        } else if (mPlatformIOS) {
          $cordovaService.toast(msg);
          $timeout(function() {
            $cordovaService.actionFinish();
          }, 1000);
        }
      };

      var _dispatchNetworkErrTag = function() {
        var tag = angular.element("#tag_err");
        if ($scope.inited && $scope.mNetworkErr) {
          tag.show();
        } else {
          tag.hide();
        }
      };

      $scope.descToHtml = function(index, desc) {
        var str = "<span class='gallery-index'>" + index + "</span><span class='gallery-count'>/" + $scope.mCurrNews.images.length + "</span> ";
        if (desc) {
          str += desc;
        }
        return str;
      };

      $scope.layoutClass = 'app-background';

      /**
       * 获取 互动评论 回调
       * @param statusCode
       */
      var interactionsCounterCallback = function(statusCode) {
        var newsID = $scope.mCurrNews.id;
        var news = findCategoryNews(newsID);
        if (news !== undefined) {
          $scope.mCurrNews.praiseNum = news.praiseNum;
          $scope.mCurrNews.commentsNum = news.commentsNum;
          $scope.mCurrNews.isPraise = news.isPraise;
        }
        $scope.$digest();
        praiseCount = 0;
      };

      function handlePoll(str) {
        var re = /<a data-cmp="poll" data-id="(.+?)">(.+?)<\/a>/g;

        return str.replace(re, function(_, id, text) {
          return '<a href="cmp://com.nd.social.vote/VoteDetail?voteId=' + id + '">' + text + '</a>';
        });
      }

      function handleActivity(str) {
        var re = /<a data-cmp="activity" data-id="(.+?)">(.+?)<\/a>/g;

        return str.replace(re, function(_, id, text) {
          return '<a href="cmp://com.nd.social.activity/actDetail?actId=' + id + '">' + text + '</a>';
        });
      }

      function handleVideo(str) {
        var re = /<a data-cmp="txvideo" data-id="(.+?)">.+?<\/a>/g;

        return str.replace(re, function(_, id) {
          return '<iframe frameborder="0" width="360" height="280" src="http://v.qq.com/iframe/player.html?vid=' + id + '&tiny=0&auto=0" allowfullscreen></iframe>';
        });
      }

      $scope.convertAsTrust = function(str) {
        // 处理投票
        str = handlePoll(str);
        // 处理活动
        str = handleActivity(str);
        // 处理视频
        str = handleVideo(str);

        return $sce.trustAsHtml(str);
      };

      /**
       * 根据 当前点赞状态 获取当前点赞图标
       * @param news
       * @returns {string}
       */
      $scope.getPraiseIcon = function() {
        if (mCurrentDetailNews.isPraise) {
          return './images/like_click_32.png';
        } else {
          return './images/like_32.png';
        }
      };

      /**
       * 判断 是否屏蔽评论和赞
       * @param news
       * @returns string
       */
      $scope.isShowComments = function() {
        return isShowComments();
      };

      $scope.isShowShare = function() {
        return isShowShare();
      };

      /**
       * 获取点赞数目
       */
      $scope.getPraiseNumber = function() {
        getNewsInteractionsCounter(getCurrentDetailNewsID(), interactionsCounterCallback);
      };

      /**
       * 触发 点赞事件
       * @param newsID
       */
      var praiseCount = 0;
      $scope.invokePraiseNews = function(newsID, event) {
        if (praiseCount === 1) {
          return;
        }

        praiseCount += 1;
        if (mCurrentDetailNews.isPraise) {
          // 新闻已经被点赞过，取消点赞
          cancelPraiseNews(newsID, function(statusCode) {
            if (SUCCESS === statusCode) {
              getNewsInteractionsCounter(newsID, interactionsCounterCallback);
              $cordovaService.refreshInteraction();
            } else {
              praiseCount = 0;
            }
          });
        } else {
          // 新闻没有被点赞过，可以点赞
          praiseNews(newsID, function(statusCode) {
            if (SUCCESS === statusCode) {
              $scope.toast($filter('translate')("news_detail_normal_js_verygood"));
              getNewsInteractionsCounter(newsID, interactionsCounterCallback);
              $cordovaService.refreshInteraction();

              if (userinfo) {
                analyticsEvent('social_news_praise', analyticsParam);
              }
            } else {
              praiseCount = 0;
            }
          });
        }
      };

      $scope.toNewsComments = function() {
        toPageComments(getCurrentDetailNewsID());
      };

      /**
       * 分享功能
       */
      $scope.share = function(news) {
        /* 已 summary 为摘要值
        var content = news.content == null ? '' : news.content;
        content = ['<div>',content,'</div>'].join('');//包裹在 div 容器里，方便后面去纯文本（踢除HTML标签）
        content =  $(content).text().substring(0,300).substring(0,300);
        */
        var summary = news.summary == null ? '' : news.summary;
        if (mPlatformAndroid) {
          ShareBox.open(categoryID, news.id, news.title, summary, news.thumbnail);
        } else if (mPlatformIOS) {
          MUICordovaWebJSExport.socialSharePresentSheetView(categoryID, news.id, news.title, summary, news.thumbnail);
          //MUICordovaWebJSExport.socialSharePresentSheetView(categoryID, news.id, news.title, news.thumbnail);
        }
      };

      /**
       * 判断 是否要显示 新闻翻页按钮
       * @returns {boolean}
       */
      $scope.showFlipButtonIfNeed = function() {
        return showFlipButtonIfNeed();
      };

      /**
       * 获取 上一条新闻标题
       * @returns {*}
       */
      $scope.getPreviousNewsTitle = function() {
        return getPreviousNewsTitle();
      };

      /**
       * 获取 下一条新闻标题
       * @returns {*}
       */
      $scope.getNextNewsTitle = function() {
        return getNextNewsTitle();
      };

      /**
       * 获取 下一条新闻的标题色值
       * @returns {*}
       */
      $scope.getNextNewsTitleColor = function() {
        return getNextNewsTitleColor();
      };
      /**
       * 获取 上一条新闻的标题色值
       * @returns {*}
       */
      $scope.getPreviousNewsTitleColor = function() {
        return getPreviousNewsTitleColor();
      };

      /**
       * 跳转 上一条新闻
       */
      $scope.invokePreviousNews = function() {
        if (!hasPreviousNews()) return;

        var newsID = getPreviousNewsID();
        if (newsID !== undefined) {
          $scope.reload(newsID);
        }
      };

      /**
       * 跳转 下一条新闻
       */
      $scope.invokeNextNews = function() {
        if (!hasNextNews()) return;

        var newsID = getNextNewsID();
        if (newsID !== undefined) {
          $scope.reload(newsID);
        }
      };

      $scope.goBigPicView = function(url) {
        goBigPicView(url);
      };

      $scope.reload = function(newsID) {
        $scope.inited = false;
        $scope.mNetworkErr = false;
        $scope.layoutClass = 'app-background';

        getNewsDetail(newsID, function(statusCode) {

          $cordovaService.refreshInteraction();
          $scope.inited = true;
          $scope.mNetworkErr = statusCode === FAILURE;

          if (SUCCESS === statusCode) {
            setCurrentDetailNewsID(newsID);
            // 当前 新闻详情的新闻对象
            $scope.mCurrNews = mCurrentDetailNews;
            $scope.publishTime = new Date($scope.mCurrNews.publish_time).format(DEFAULT_TIME_FORMAT);
            $scope.getPraiseNumber();
          } else if (FAILURE === statusCode) {

          } else if (DELETED === statusCode) {
            $cordovaService.toastAndFinish($filter('translate')("news_detail_normal_js_already_delete"));

            var deleteMark = restoreLocalJsonData(DELETE_MARK);
            if (deleteMark === undefined) {
              deleteMark = [];
              deleteMark.push(newsID);
            } else {
              deleteMark.push(newsID);
            }
            localStorage.setItem(DELETE_MARK, JSON.stringify(deleteMark));

          }

          _dispatchNetworkErrTag();
        });
      };

      var needInvokeRefresh = false;
      var _invokeRefreshIfNeed = function() {
        if (needInvokeRefresh) return;

        $cordovaService.refreshInteraction();
        needInvokeRefresh = true;
      };

      /****************************************************************************/
      window.translate = function(msg) {
        return $filter('translate')(msg);
      };

      window.onWebviewWillUpdate = function() {
        if ($scope.mCurrNews !== undefined) {
          getNewsInteractionsCounter($scope.mCurrNews.id, interactionsCounterCallback);
        }
      };

      window.logInEven = function(uid) {
        if (uid) {
          var ndpcSuccess = 'getCurrentUserSuccess';
          var ndpcFail = 'getCurrentUserFail';

          //获取用户信息的成功回调
          window[ndpcSuccess] = function(data) {
            userinfo = JSON.stringify(data);

            if (userinfo) {
              var ipArea = getIpArea(userinfo);

              analyticsParam = {
                'categoryId': categoryID,
                'newsId': newsID,
                'province': ipArea[0],
                'city': ipArea[1]
              };
            }

            if (mPlatformAndroid) {
              analyticsParam = JSON.stringify(analyticsParam);
            }
          };
          //获取用户信息的失败回调
          window[ndpcFail] = function(data) {
            console.log("get user info error");
          };

          if (mPlatformAndroid) {
            $cordovaService.refreshInteraction();
            JsHttpRequest.getCurrentUser(ndpcSuccess, ndpcFail);
          }
          if (mPlatformIOS) {
            MUICordovaWebJSExport.getUserInfo(ndpcSuccess, ndpcFail);
          }
        }
      };
    }
  ]);
