// 新闻列表 控制器
angular.module('news.controller.news_list', ['news.service.cordova', 'i18n_module'])
  .controller('NewsListCtrl', ['$rootScope', '$scope', '$timeout', '$location', '$sce', '$element', 'cordovaService', '$translate', '$filter',
    function($rootScope, $scope, $timeout, $location, $sce, $element, $cordovaService, $translate, $filter) {

      function initWidth() {
        $scope.mScrollerHeight = document.documentElement.clientHeight;
        // PMS #29126
        $scope.mAdWidth = document.documentElement.clientWidth;
        $scope.mAdHeight = Math.round($scope.mAdWidth * 10 / 17);
      }

      initWidth();

      $scope.mNetworkErr = false;
      $scope.isPullDown = false;

      registerEventBus("NewsListCtrl_element_refresh", function(event) {
        if (EVENT_ELEMENT_REFRESH === event) {
          $scope.$digest();
        }
      });

      var requestLatestNewsCallback = function(category, statusCode) {
        if (SUCCESS === statusCode) {
          if (category.appendSize > 0) {
            if ($scope.isPullDown) {
              $scope.toast($filter('translate')('main_js_add_x_news', '{value:' + category.appendSize + '}'));
            }
          } else {
            if ($scope.isPullDown) {
              $scope.toast($filter('translate')('news_list_js_no_more_news'));
            }
          }
        } else if (EMPTY === statusCode) {
          $scope.toast($filter('translate')('news_list_js_no_more_news'));
        }

        if (statusCode === 600) {
          $scope.toast($filter('translate')('news_list_network_anomaly'));
        }

        $scope.mNetworkErr = statusCode === FAILURE;
        $scope.$digest();
        slide();

        //下面可能会出错
        $cordovaService.donePulldown();

      };

      var requestOlderNewsCallback = function(category, statusCode, size) {
        if (statusCode === EMPTY && !isCategoryHasOlderNews(category)) {
          $scope.toast($filter('translate')('news_list_js_no_more_news'));
        }

        if (SUCCESS === statusCode) {}

        // 延时调用，给DOM渲染时间
        $timeout(function() {
          $cordovaService.donePullup(size);
        }, 400);

        $scope.$digest();
      };

      /**
       * 判断 新闻分类 是否有进行过新闻请求
       * @param category
       * @returns {boolean}
       */
      $scope.isCategoryNewsRequested = function(category) {
        return isCategoryNewsRequested(category);
      };

      /**
       * 判断 新闻分类 是否包含新闻内容
       * @param category
       * @returns {*}
       */
      $scope.isCategoryHasNews = function(category) {
        return isCategoryHasNews(category);
      };

      /**
       * 获取新闻时间
       * @param news
       * @returns string
       */
      $scope.getNewsTime = function(news) {
        return timeFormat(news);
      };

      /**
       * 判断 是否屏蔽评论和赞
       * @param news
       * @returns string
       */
      $scope.isShowComments = function() {
        return isShowComments();
      };

      /**
       * 获取评论数
       * @param news
       * @returns string
       */
      $scope.getComments = function(news) {
        return news.commentsNum;
      };

      /**
       * 文本溢出省略号
       */
      $scope.clamp = function() {
        clamp();
      };

      /**
       * 判断 新闻分类 是否还有较旧的数据
       * @param categoryId
       * @returns {*}
       */
      $scope.isCategoryHasOlderNews = function(category) {
        return isCategoryHasOlderNews(category);
      };

      /**
       * 加载 较后面的数据
       * @param categoryId
       */
      $scope.getOlderNewsByCategory = function(category) {
        getCategoryNewsOlder(category, DEFAULT_REQUEST_SIZE, function(category, statusCode, size) {
          requestOlderNewsCallback(category, statusCode, size);
        });
      };

      /**
       * 获取上次更新时间
       * @param category
       * @returns {*}
       */
      $scope.getLastRequestTime = function(category) {
        var time = getLastUpdateNewsTime(category);
        if (time === undefined) {
          time = $filter('translate')('no_more_items');
        }
        return time;
      };

      /**
       * 请求最新的 分类新闻
       * @param categoryId
       */
      $scope.getLatestNewsByCategory = function(category) {
        $scope.mNetworkErr = false;
        $scope.$digest();

        getCategoryNewsLatest(category, DEFAULT_REQUEST_SIZE, function(category, statusCode) {
          requestLatestNewsCallback(category, statusCode);
        });

        updateNewsInteractionsCounterByCategory(category, function(statusCode) {
          $scope.$digest();
        });
      };

      /**
       * 判断 新闻是否被阅读过
       * @param news
       * @returns {News.isRead|*}
       */
      $scope.isNewsHasRead = function(news) {
        return hasNewsRead(news);
      };

      /**
       * 根据新闻已看情况获取样式
       * @param news
       * @returns {string}
       */
      $scope.getClassNameByRead = function(news) {
        if ($scope.isNewsHasRead(news)) {
          return 'app-text-read';
        } else {
          return 'app-text';
        }
      };

      /**
       * 获取新闻热度
       * @param news
       * @returns {*}
       */
      $scope.isNewsHot = function(news) {
        return getNewsHot(news) >= HOT_NUM;
      };

      /**
       * 判断 是否是图文新闻
       * @param type
       * @returns {boolean}
       */
      $scope.isTypeOfNews = function(news) {
        return isTypeOfNews(news);
      };

      /**
       * 判断 是否是图集新闻
       * @param type
       * @returns {boolean}
       */
      $scope.isTypeOfGallery = function(news) {
        return isTypeOfGallery(news);
      };

      /**
       * 判断 是否是专题
       * @param news
       */
      $scope.isTypeOfTopic = function(news) {
        return isTypeOfTopic(news);
      };

      /**
       * 触发 新闻点击
       * @param news
       */
      $scope.invokeNewsDetail = function(categoryId, news, $element) {
        markNewsHasRead(news);

        if ($scope.isTypeOfTopic(news)) {
          toPageTopicDetail(categoryId, news.topics[0]);
        } else if ($scope.isTypeOfGallery(news)) {
          news.hits += 1;
          toNewsDetailGallery(categoryId, news.id, 0);
        } else {
          news.hits += 1;
          toNewsDetailNormal(categoryId, news.id);
        }
      };

      /**
       * 进入 新闻详情页面 并携带翻页（用于图集）
       * @param categoryId
       * @param news
       * @param slide
       */
      $scope.invokeNewsDetailWidthSlide = function(categoryId, news, slide, $event) {
        $event.stopPropagation(); // 阻止 事件冒泡

        if (!$scope.isTypeOfTopic(news)) {
          toNewsDetailGallery(categoryId, news.id, slide);
        } else {
          $scope.invokeNewsDetail(categoryId, news);
        }
      };

      $scope.toast = function(msg) {
        //FastToast( msg );
        $cordovaService.toast(msg);
      };

      /**
       * 根据新闻ID 删除新闻
       * @param newsID
       */
      function _deleteNews(newsID) {
        for (var i = 0, nmax = $scope.category.newsArray.length; i < nmax; i++) {
          if ($scope.category.newsArray[i].id == newsID) {
            $scope.category.newsArray.splice(i, 1);
          }
        }
        for (var j = 0, cnmax = mCategoryNews.length; j < cnmax; j++) {
          if (mCategoryNews[j].id == newsID) {
            mCategoryNews.splice(j, 1);
          }
        }
        $scope.category.hasNews = $scope.category.newsArray.length + $scope.category.thumbNews.length > 0;
      }

      $scope.inited = false;
      $scope.offline = false;
      $scope.pull = false;

      function initData(categoryId) {
        if (!$scope.mAdWidth) {
          initWidth();
          return setTimeout(function() {
            initData(categoryId);
          }, 100);
        }
        $scope.offline = false;
        $scope.inited = true;
        $scope.mNetworkErr = false;
        $scope.pull = false;

        $scope.category = new Category(categoryId);

        setCurrentCategoryID(categoryId);
        setCurrentCategory($scope.category);

        getCategoryNewsLatest($scope.category, DEFAULT_REQUEST_SIZE, function(category, statusCode) {
          requestLatestNewsCallback(category, statusCode);
        });

        if (mCordovaSupport) {
          if (mCordovaReady) {
            if (!$scope.offline && !$scope.pull) {
              $cordovaService.addPullRefreshForIos();
            }
          } else {
            registerEventBus("NewsListCtrl_addPullRefreshForIos", function(event) {
              if (EVENT_CORDOVA_READY === event) {
                if (!$scope.offline && !$scope.pull) {
                  $cordovaService.addPullRefreshForIos();
                }
              }
            });
          }
        }
      }

      /**
       * 初始化
       */
      $scope.init = function() {
        initData(CID);
      };

      /*****************************************************************/
      window.translate = function(msg) {
        return $filter('translate')(msg);
      };

      window.startPullDownToRefresh = function() {
        var time = $filter('translate')('refresh_time') + $scope.getLastRequestTime($scope.category);
        if (mPlatformAndroid) {
          webView.setRefreshTime(time);
        } else if (mPlatformIOS) {
          $cordovaService.setRefreshTimeForIos(time);
        }
      };

      window.onPullDownToRefresh = function() {
        if ($scope.offline) {
          $scope.pull = true;
          initData();
        }

        $scope.isPullDown = true;
        $scope.getLatestNewsByCategory($scope.category);
      };

      window.onPullUpToRefresh = function() {
        if ($scope.offline) {
          $scope.pull = true;
          initData();
        }

        $scope.getOlderNewsByCategory($scope.category);
      };

      function _onCategoryReload(categoryId) {
        if (!$scope.mAdWidth) {
          initWidth();
          return setTimeout(function() {
            _onCategoryReload(categoryId);
          }, 100);
        }
        window.scrollTo(0, 0);
        if (categoryId !== undefined) {
          mCategoryNews.length = 0;
          $scope.isPullDown = false;
          $scope.inited = false;
          $scope.category = new Category(categoryId);

          setCurrentCategoryID(categoryId);
          setCurrentCategory($scope.category);

          $scope.mNetworkErr = false;
          $scope.$digest();

          getCategoryNewsLatest($scope.category, DEFAULT_REQUEST_SIZE, function(category, statusCode) {
            $scope.inited = true;
            requestLatestNewsCallback(category, statusCode);
          });
        }
      }

      window.onCategoryReload = function(categoryId) {
        _onCategoryReload(categoryId);
      };

      window.onWebviewWillUpdate = function() {

        if ($scope.inited) {

          if (mSupportLocalStorage) {
            var restore = restoreLocalJsonData(DELETE_MARK);
            if (restore !== undefined) {
              for (var i = 0, max = restore.length; i < max; i++) {
                _deleteNews(restore[i].id);
              }
            }
          }

          if ($scope.category !== undefined) {
            updateNewsInteractionsCounterByCategory($scope.category, function(statusCode) {
              $scope.$digest();
            });
          }

          $scope.$digest();
        }
      };

    }
  ]);
