// 专题详情页 控制器
angular.module('news.controller.topic', ['news.service.cordova', 'i18n_module'])
  .controller('TopicPageCtrl', ['$rootScope', '$scope', '$timeout', '$location', '$sce', '$translate', 'cordovaService', '$filter',
    function($rootScope, $scope, $timeout, $location, $sce, $translate, $cordovaService, $filter) {

      $scope.mAdWidth = document.documentElement.clientWidth;
      $scope.mAdHeight = $scope.mAdWidth * 1 / 3;

      registerEventBus("TopicPageCtrl_element_refresh", function(event) {
        if (EVENT_ELEMENT_REFRESH === event) {
          $scope.$digest();
        }
      });

      $scope.mTopicID = $.getUrlParam('topicid');
      $scope.mCateID = $.getUrlParam('categoryid');

      /**
       * 判断 新闻是否被阅读过
       * @param news
       * @returns {News.isRead|*}
       */
      $scope.isNewsHasRead = function(news) {
        return hasNewsRead(news);
      };

      /**
       * 根据新闻已看情况获取样式
       * @param news
       * @returns {string}
       */
      $scope.getClassNameByRead = function(news) {
        if ($scope.isNewsHasRead(news)) {
          return 'app-text-read';
        } else {
          return 'app-text';
        }
      };

      /**
       * 获取新闻时间
       * @param news
       * @returns string
       */
      $scope.getNewsTime = function(news) {
        return timeFormat(news);
      };

      /**
       * 判断 是否屏蔽评论和赞
       * @param news
       * @returns string
       */
      $scope.isShowComments = function() {
        return isShowComments();
      };

      /**
       * 获取评论数
       * @param news
       * @returns string
       */
      $scope.getComments = function(news) {
        return news.commentsNum;
      };

      /**
       * 获取新闻热度
       * @param news
       * @returns {*}
       */
      $scope.isNewsHot = function(news) {
        return getNewsHot(news) >= HOT_NUM;
      };

      /**
       * 判断 是否是图文新闻
       * @param type
       * @returns {boolean}
       */
      $scope.isTypeOfNews = function(news) {
        return isTypeOfNews(news);
      };

      /**
       * 判断 是否是图集新闻
       * @param type
       * @returns {boolean}
       */
      $scope.isTypeOfGallery = function(news) {
        return isTypeOfGallery(news);
      };

      /**
       * 文本溢出省略号
       */
      $scope.clamp = function() {
        clamp();
      };

      /**
       * 触发 新闻点击
       * @param news
       */
      $scope.invokeNewsDetail = function(news) {
        markNewsHasRead(news);

        if ($scope.isTypeOfGallery(news)) {
          news.hits += 1;
          toNewsDetailGallery($scope.mCateID, news.id, 0);
        } else {
          news.hits += 1;                      
          $scope.openNewsDetail(news.id);
        }
      };

      /**
       * 进入 新闻详情页面
       * @param newID
       */
      $scope.openNewsDetail = function(newsID) {
        // 专题新闻打开时分类ID为null
        toTopicNewsDetailNormal($scope.mCateID, $scope.mTopicID, newsID);
      };

      /**
       * 进入 新闻详情页面 并携带翻页（用于图集）
       * @param categoryID
       * @param news
       * @param slide
       */
      $scope.invokeNewsDetailWidthSlide = function(news, slide, $event) {
        $event.stopPropagation(); // 阻止 事件冒泡
        toNewsDetailGallery($scope.mCateID, news.id, slide);
      };

      /*-----------------------------------------------------------------------------------*/

      $scope.isTopicHasOlderNews = function() {
        return isCurrentTopicHasOlderNews();
      };

      $scope.getLastRequestTime = function() {
        var time = getTopicNewsLastUpdateTime();
        if (time === undefined) {
          time = $filter('translate')('no_more_items');
        }
        return time;
      };

      $scope.getTopicSummary = function() {
        return $sce.trustAsHtml('<span class="topic-detail-summary">' + $filter('translate')('topic_detail_topic') + '</span>' + $scope.mTopic.summary);
      };

      var getLatestNewsCallback = function(statusCode) {
        $scope.inited = true;
        if (FAILURE != statusCode) {
          $scope.mTopicNews = getCurrentTopic().newsArray;
        }

        if (EMPTY === statusCode) {
          if (isCurrentTopicHasNews) {
            $scope.toast($filter('translate')('topic_detail_no_more_news'));
          }
        } else if (SUCCESS === statusCode) {
          /*if (mCurrentDetailTopic.appendSize > 0) {
            $scope.toast($filter('translate')('main_js_add_x_news', '{value:' + mCurrentDetailTopic.appendSize + '}'));
          }*/
        }

        $scope.$digest();

        $cordovaService.donePulldown();
      };

      var getOlderNewsCallback = function(statusCode, size) {
        $scope.inited = true;

        if (FAILURE != statusCode) {
          $scope.mTopicNews = getCurrentTopic().newsArray;
          if (!$scope.isTopicHasOlderNews()) {
            $scope.toast($filter('translate')('topic_detail_no_more_news'));
          }
        }

        $scope.$digest();

        $cordovaService.donePullup(size);
      };

      $scope.getTopicNewsLatest = function() {
        var topicID = $.getUrlParam('topicid');
        setCurrentTopicID(topicID);

        getTopicNewsLatest(topicID, DEFAULT_REQUEST_SIZE, function(statusCode) {
          getLatestNewsCallback(statusCode);
        });

        updateNewsInteractionsCounterByTopic(topicID, function(statusCode) {
          $scope.$digest();
        });
      };

      $scope.getTopicNewsOlder = function() {
        getTopicNewsOlder(getCurrentTopicID(), DEFAULT_REQUEST_SIZE, function(statusCode, size) {
          getOlderNewsCallback(statusCode, size);
        });
      };

      $scope.toast = function(msg) {
        $cordovaService.toast(msg);
      };

      $scope.inited = false;
      $scope.mNetworkErr = false;

      $scope.init = function() {
        var topicID = $.getUrlParam('topicid');

        if (topicID !== undefined) {
          setCurrentTopicID(topicID);

          getTopicDetail(topicID,
            function(statusCode) {
              $scope.inited = true;
              $scope.mNetworkErr = statusCode === FAILURE;

              if (FAILURE != statusCode) {
                $scope.mTopic = mCurrentDetailTopic;
                $scope.mTopicNews = mCurrentDetailTopic.newsArray;
              }

              _dispatchNetworkErrTag();

              if (SUCCESS === statusCode) {
                // 获取最新的专题新闻
                getTopicNewsLatest(topicID, DEFAULT_REQUEST_SIZE, function(statusCode) {
                  getLatestNewsCallback(statusCode);
                });
              }

              $scope.$digest();
            });

          if (mCordovaSupport) {
            if (mCordovaReady) {
              $cordovaService.addPullRefreshForIos();
            } else {
              registerEventBus("TopicPageCtrl_add_pull_refresh", function(event) {
                if (EVENT_CORDOVA_READY === event) {
                  $cordovaService.addPullRefreshForIos();
                }
              });
            }
          }

        }
      };

      var _dispatchNetworkErrTag = function() {
        var tag_err = angular.element("#tag_err");
        var tag_loading = angular.element("#tag_loading");
        if ($scope.inited && $scope.mNetworkErr) {
          tag_err.show();
          tag_loading.hide();
        } else {
          tag_err.hide();
        }
      };

      /**
       * 根据新闻ID 删除新闻
       * @param newsID
       */
      function _deleteNews(newsID) {
        for (var i = 0; i < $scope.mTopicNews.length; i++) {
          if ($scope.mTopicNews[i].id == newsID) {
            $scope.mTopicNews.splice(i, 1);
          }
        }
        for (var i = 0; i < mCategoryNews.length; i++) {
          if (mCategoryNews[i].id == newsID) {
            mCategoryNews.splice(i, 1);
          }
        }
      }

      /*****************************************************************/

      window.translate = function(msg) {
        return $filter('translate')(msg);
      };

      window.startPullDownToRefresh = function() {
        var time = $filter('translate')('refresh_time') + $scope.getLastRequestTime();
        if (mPlatformAndroid) {
          webView.setRefreshTime(time);
        } else if (mPlatformIOS) {
          $cordovaService.setRefreshTimeForIos(time);
        }

      };

      window.onPullDownToRefresh = function() {
        var topicID = $.getUrlParam('topicid');
                                
        if (topicID !== undefined) {
          getTopicDetail(topicID, function(statusCode) {
            getTopicNewsLatest(topicID, DEFAULT_REQUEST_SIZE, function(statusCode) {
              getLatestNewsCallback(statusCode);
              $scope.mTopic = mCurrentDetailTopic;
            });
            
          });
        }
         
                               
      };

      window.onPullUpToRefresh = function() {
        $scope.getTopicNewsOlder();
      };

      window.onWebviewWillUpdate = function() {
        if ($scope.inited) {

          if (mSupportLocalStorage) {
            var restore = restoreLocalJsonData(DELETE_MARK);
            if (restore !== undefined) {
              for (var i = 0; i < restore.length; i++) {
                _deleteNews(restore[i].id);
              }
            }
          }

          if ($scope.mTopic !== undefined) {
            updateNewsInteractionsCounterByTopic($scope.mTopic.id, function(statusCode) {
              $scope.$digest();
            });
          }

          $scope.$digest();
        }

      };

    }
  ]);
