(function($){
	/*
	 * 常量
	 */
	var constants = {
		param:{
			cookie_session:'content_session',
			content_param_file:'file', //内容服务器文件参数名称
			content_param_path:'path' //要存储的文件路径
		},
		server:{
			content_service_url:(function(){
				return localStorage.getItem(KEY_STORE_CS_API);
			}()),//内容服务器地址
			get_session_url:'contents/sessions',	//服务端获取session URL
			upload_url:'upload', //上传图片
			get_image_url:'static' //path方式下载图片
		},
		defaultSetting:{
			'scope':1
		}
	};

	/*
	 * 初始化上传文件参数
	 */
	var initFileParam = function(settings,error){
		var files = $('#' + settings.fileId)[0].files;
		var formData = new FormData();
		for(var i = 0; i < files.length; ++i){
			formData.append(constants.param.content_param_file, files[i]);
		}

		settings = $.extend(settings,constants.defaultSetting);

		//初始化其他一些参数
		for(var pro in settings){
			if(pro != 'fileId'){
				formData.append(pro,settings[pro]);
			}
		}

		return formData;
	};

	/*
	 * 回调
	 */
	var message = function(fn,message){
		var result = {};
		result.message = message;
		if(fn != undefined){
			fn(result);
		}
	};

	/*
	 * 包装请求时处理session过期
	 */
	var contentAjax = function(settings,fn,args){
		var errFunc = settings.error;
		var onProgressFunc = settings.onProgress;
		var _setting = {
			'error' : function(responseStr){
				if(responseStr.status != undefined
		    			&& responseStr.status == '403'){
		    		//session 过期
		    		$.setCookie(constants.param.cookie_session,'');
		    		message(onProgressFunc,'session 过期');
		    		$.sessionFilter(function(cookie){
		    			if(fn != undefined){
		    				if(args != undefined && args.length > 0){
		    					var s = "fn(";
		    					for(var i = 0; i < args.length; ++i){
		    						s += "args[" + i + "],"
		    					}
		    					s = s.substring(0,s.length - 1) + ")";
		    					eval(s);
		    				}
		    			}
		    		},settings.error,settings.onProgress);
		    		return;
		    	}
				if(errFunc != undefined)
					errFunc(responseStr);
			}
		};

		settings = $.extend(settings,_setting);
		$.ajax(settings);
	};

	/*
	 * 解析cookie
	 */
	var getCookieParam = function(param){
		var cookie = $.getCookie(constants.param.cookie_session);
		if(cookie == null)
			return undefined;
		var params = cookie.split(",");
		if(param == 'session')
			return params[0];
		if(param == 'path')
			return params[1];
		return undefined;
	};

	$.extend({
		/*
		 * 上传
		 */
		upload:function(settings,success,error,onProgress){
			$.sessionFilter(function(cookie){
				if($('#' + settings.fileId) == undefined){
					message(error,'fileId is undefine');
					return;
				}
				var files = $('#' + settings.fileId)[0].files;
				if(files == undefined || files.length <= 0){
					message(error,'no selected any files');
					return;
				}
				var path = getCookieParam('path');
				settings.path = path;
				message(onProgress,'开始上传');
				var formData = initFileParam(settings,error);
				var session = getCookieParam('session');
				var url = constants.server.content_service_url
							+ constants.server.upload_url + '?session=' + session;
				var args = [settings,success,error,onProgress];
				contentAjax({
		            url : url,
		            type : 'POST',
		            processData : false,
		            contentType : false,
		            data : formData,
		            success : function(responseStr) {
		            	message(onProgress,'文件上传成功');
		            	success(responseStr);
		            },
		            error : function(responseStr) {
		            	message(error,'文件上传失败');
		            	if(responseStr.status != undefined
		            			&& responseStr.status == '403'){
		            		//session 过期
		            		message(onProgress,'session 失效，重新获取session');
		            		$.startUpload(settings,success,error,onProgress);
		            		return;
		            	}
		            	error(responseStr);
		            },
		            onProgress:onProgress
		        },$.upload,args);
			},error,onProgress);
		},
		/*
		 * 编码URL
		 */
		encodeContentUrl:function(fileName,size,success,error,onProgress){
			$.sessionFilter(function(cookie){
				var path = getCookieParam('path');
				var url = constants.server.content_service_url
					+ constants.server.get_image_url + path + '/' + fileName + '?size=' + size;
				success(url);
			},error,onProgress);
		},
		/*
		 * 拼凑URL
		 * 请求内容服务删除文件
		 */
		deleteFile:function(filePath,success,error,onProgress){
			$.sessionFilter(function(cookie){
				var session = getCookieParam('session');
				var path = getCookieParam('path');
				var url = constants.server.content_service_url
					+ constants.server.get_image_url + path + '/' + filePath + '?session=' + session;
				var args = [filePath,success,error,onProgress];
				contentAjax({
		            url : url,
		            type : 'DELETE',
		            success : function(responseStr) {
		            	message(onProgress,'文件删除成功');
		            	success(responseStr);
		            },
		            'error':error,
		            'onProgress':onProgress
	            },$.deleteFile,args);
			},error,onProgress);
		},
		/*
		 * 获取session
		 */
		getSession:function(success,error,onProgress){
			message(onProgress,'开始获取session');
			$.getJSON(constants.server.get_session_url,{},
				function(data){
					message(onProgress,'获取session成功');
					var cookie = data.session + "," + data.path;
					$.setCookie(constants.param.cookie_session,cookie);
					success(cookie);
				},
				function(response){
					message(onProgress,'获取session失败，可能还未登陆');
					message(error,'get Session error');
					error(response);
			});
		},
		/*
		 * session过滤器
		 */
		sessionFilter:function(fn,error,onProgress){
			var cookie = $.getCookie(constants.param.cookie_session);
			if(cookie == null
					|| cookie == ''){
				$.getSession(function(cookie){
					if(fn != undefined)
						fn(cookie);
				},error,onProgress);
				return;
			}
			if(fn != undefined)
				fn(cookie);
		}
	});

})(jQuery);
