/**
 * 自动登陆
 * @param successCallback
 * @param failureCallback
 */
// window.onerror = function(e,f,r,c) {alert([e,f,r,c].join('\n'))}
var uidKey = $.getUrlParam('uid'),
  categoriesKey = uidKey + '_' + KEY_STORE_CATEGORIES,
  newsKey = uidKey + '_' + KEY_STORE_NEWS;

function autoLogin(successCallback, failureCallback) {
  $.postJSON('/users/actions/login', {
    login_name: '515253',
    password: 'asdfasdf'
  }, function(data) {
    $.setCookie('access_token', data.access_token);
    $.setCookie('mac_key', data.mac_key);

    if (successCallback !== undefined) {
      successCallback(data);
    }

  }, function(data) {
    log("login failure !!");
    log(data);

    if (failureCallback !== undefined) {
      failureCallback(data);
    }
  });
}

/**
 * 选择主题
 * @param selectValue
 */
function selectTheme(selectValue) {
  $.each(THEMES, function(n, value) {
    if (selectValue === value) {
      $("link[title='" + value + "']").removeAttr("disabled");
    } else {
      $("link[title='" + value + "']").attr("disabled", "disabled");
    }
  });
}

/**
 * 选择字体
 * @param selectValue
 */
function selectFont(selectValue) {
  $.each(FONTS, function(n, value) {
    if (selectValue + "_font" === value) {
      $("link[title='" + value + "']").removeAttr("disabled");
    } else {
      $("link[title='" + value + "']").attr("disabled", "disabled");
    }
  });


}

/*------------------------------------------------------------------------------*/

/**
 * 服务器上的设置
 */
var severPersonalization;


/**
 * 初始化
 * @param callback
 */
function init(callback) {
  //获取个性设置
  getPersonalization();
  //获取版块列表
  getCategories(callback);
}


/**
 * 获取个性设置
 * @param callback 回调
 */
function getPersonalization() {
  $.getJSON("/news/personalization", {}, function(data) {
    if (data) {
      severPersonalization = data;
      if (data.theme) {
        selectTheme(data.theme);
      }
      if (data.font) {
        selectFont(data.font);
      }
      //if(data.font){
      //}
    }
  }, function(data, statusCode) {
    selectTheme(THEMES[0]);
    selectFont(FONTS[0]);
  });
}


/*------------------------------------------------------------------------------*/

var mRestoredCategories = false; // 是否缓存恢复 新闻分类
var mRestoredNews = false; // 是否缓存恢复 新闻

function restoreLocalJsonData(key) {
  if (mSupportLocalStorage) {
    var data;
    if (mPlatformAndroid) {
      data = JsSaveSp.getItem(key);
    }
    if (mPlatformIOS) {
      data = localStorage.getItem(key);
    }
    if (data) {
      var json = JSON.parse(data);
      if (json !== undefined) {
        return json;
      }
    }
  }

  return undefined;
}


/*************************************************************************************/

function buildCategory(data) {
  var category = new Category(data.id);
  updateCategory(data, category);
  return category;
}

function updateCategory(src, dest) {
  if (src === undefined || dest === undefined) return;
  dest.desc = src.desc;
  dest.name = src.name;
  dest.sort = src.sort;
}

/**
 * 新闻的分类
 * @type {Array}
 */
var mCategories = [];

/**
 * 恢复 本地缓存的 新闻分类数据
 * @returns {*}
 */
function restoreCategoriesData() {
  if (mSupportLocalStorage) {
    var restore = restoreLocalJsonData(categoriesKey);
    mRestoredCategories = restore !== undefined;
    if (mRestoredCategories) {
      return restore;
    }
  }
  return [];
}

/**
 * 本地缓存 新闻分类数据
 */
function storeCategories(data) {

  if (mSupportLocalStorage) {
    if (mPlatformAndroid) {
      JsSaveSp.removeItem(categoriesKey);
      JsSaveSp.setItem(categoriesKey, JSON.stringify(data));
    }
    if (mPlatformIOS) {
      localStorage.removeItem(categoriesKey);
      localStorage.setItem(categoriesKey, JSON.stringify(data));
    }


  }
}

/**
 * 插入或更新 新闻的分类
 * @param categorys
 * @returns {*}
 */
function insertOrUpdateCategory(category) {

  if (category.id === undefined) return;

  var found = findCategory(category.id);
  if (found === undefined) {
    mCategories.push(buildCategory(category));
  } else {
    updateCategory(category, found);
  }

}

/**
 * 获取 指定的 新闻的分类
 * @param categoryID
 * @returns {*}
 */
function findCategory(categoryID) {
  return findItem(mCategories, categoryID);
}


/**
 * 获取 新闻的分类
 * @param callback
 */
function getCategories(callback) {

  $.getJSON("/categories", {
    size: 100
  }, function(data) {
    if (data === undefined || data.items.length <= 0) {
      callback(data, EMPTY);
      return;
    }

    for (var j = 0, max = data.items.length; j < max; j++) {
      var item = data.items[j];
      insertOrUpdateCategory(item);
    }

    storeCategories(data);
    callback(data, SUCCESS);
  }, function(data, statusCode) {
    // dbcache缓存分类，断网的时候取缓存
    var categories = restoreCategoriesData();
    if (categories && !data.code) {
      callback(categories, 600);
      return;
    }
    if (!categories || categories && categories.length <= 0) {
      callback(categories, EMPTY);
      return;
    }
    for (var j = 0, max = categories.length; j < max; j++) {
      var item = categories[j];
      insertOrUpdateCategory(item);
    }

    callback(categories, SUCCESS);
  });
}


/*------------------------------------------------------------------------------*/

/**
 * 当前的新闻类别ID
 * @type {number}
 */
var mCurrentCategoryID;
var mCurrentCategory;

function setCurrentCategoryID(categoryID) {
  if (categoryID === undefined) return;

  mCurrentCategoryID = categoryID;
  if (mSupportLocalStorage) {
    localStorage.setItem(KEY_STORE_CURRENT_CATEGORY_ID, categoryID);
  }
}

function getCurrentCategoryID() {
  if (mCurrentCategoryID !== undefined) return mCurrentCategoryID;

  if (mSupportLocalStorage) {
    mCurrentCategoryID = localStorage.getItem(KEY_STORE_CURRENT_CATEGORY_ID);
  }
  if (mCategories.length > 0) return mCategories[0].id;
  return mCurrentCategoryID;
}

function getCurrentCategory() {
  return mCurrentCategory;
}

function setCurrentCategory(category) {
  mCurrentCategory = category;
}

/**
 * 初始化时获取第一个分类的新闻
 * @param callback
 */
function getCategoryNewsWhenInit(callback) {
  if (mCategories.length > 0) {
    getCategoryNewsLatest(mCategories[0], DEFAULT_REQUEST_SIZE, callback);
  }
}

/**
 * 判断 分类新闻是否获取过
 * @param categoryID
 * @returns {boolean}
 */
function isCategoryNewsRequested(category) {
  if (category !== undefined) {
    return category.newsRequested;
  }
  return false;
}

/**
 * 标记 分类新闻获取过
 * @param categoryID
 */
function markCategoryNewsRequested(category) {
  if (category !== undefined) {
    category.lastUpdateNewsTime = Math.max(category.maxTs, getNowUnixTime());
    category.newsRequested = true;
  }
}

/**
 * 判断 分类对应的新闻是否为空
 * @param categoryID
 * @returns {*}
 */
function isCategoryHasNews(category) {
  if (category !== undefined) {
    return category.hasNews;
  }
  return false;
}

/**
 * 标记 分类对应的新闻不为空
 * @param categoryID
 */
function markCategoryHasNews(category) {
  if (category !== undefined) {
    category.hasNews = true;
  }
}

/**
 * 自动检测 分类对应的新闻是否为空
 * @param categoryID
 */
function checkCategoryHasNews(categoryID) {
  if (categoryID === undefined) return;

  var category = findCategory(categoryID);
  if (category !== undefined) {
    category.hasNews = category.newsArray.length > 0;
  }
}


/**
 * 判断 分类对应的新闻是否还有早期新闻需要获取
 * @param categoryID
 * @returns {*}
 */
function isCategoryHasOlderNews(category) {
  if (category !== undefined) {
    return category.moreOlderNews && category.minTs != INVALID;
  }
  return false;
}

/**
 * 标注 分类对应的新闻是否还有早期新闻需要获取
 * @param categoryID
 * @param hasOlderNews
 * @returns {*}
 */
function markCategoryHasOlderNews(category, hasOlderNews) {
  if (category !== undefined) {
    category.moreOlderNews = hasOlderNews;
  }
}

/**
 * 刷新 分类新闻的更新时间戳
 * @param categoryID
 * @param time
 */
function updateCategoryTimemark(category, news) {
  if (category !== undefined) {
    var time = news.publish_time;

    if (!news.icons.length) {
      // min
      if (INVALID === category.minTs || time < category.minTs) {
        category.minTs = time;
      }
    }

    // max
    if (INVALID === category.maxTs || time > category.maxTs) {
      category.maxTs = time;

      var max_time_to_native = category.maxTs;

      //保存 maxts 到 native/localStorage 以供 native 读取
      if (mPlatformAndroid) {
        var itemKey = 'news_max_ts_' + getCurrentUid();
        var max_time = JsSaveSp.getItem(itemKey);

        if (+max_time_to_native > +max_time) {
          JsSaveSp.setItem(itemKey, JSON.stringify(max_time_to_native));
        }
      }

      //iOS自己会比较判断这个时间以决定是否覆盖，所以不用我们比较了
      if (mPlatformIOS) {
        MUICordovaWebJSExport.saveLatestNewUpdateTime(max_time_to_native);
      }

    }
  }
}

/**
 * 获取 分类上一次请求新闻的时间
 * @param categoryID
 * @returns {*}
 */
function getLastUpdateNewsTime(category) {
  if (category !== undefined && category.lastUpdateNewsTime > 0) {
    return new Date(category.lastUpdateNewsTime).format(DEFAULT_TIME_FORMAT);
  }
  return undefined;
}

/*------------------------------------------------------------------------------*/

/**
 *  新闻
 * @type {Array}
 */
var mCategoryNews = [];
//var mCategoryNews = restoreCategoryNewsData();

/**
 * 恢复 本地缓存的 新闻分类数据
 * @returns {*}
 */
function restoreCategoryNewsData() {
  if (mSupportLocalStorage) {
    var restore = restoreLocalJsonData(KEY_STORE_NEWS);
    mRestoredNews = restore !== undefined;
    if (mRestoredNews) {
      return restore;
    }
  }
  return [];
}

function buildNews(data) {
  var news = new News(data.id);
  updateNewsBase(data, news);
  return news;
}

/**
 * 更新 新闻基本信息
 * @param src
 * @param dest
 */
function updateNewsBase(src, dest) {
  if (src === undefined || dest === undefined) return;
  dest.categories = src.categories;
  dest.thumbnail = src.thumbnail;
  dest.images = src.images;
  dest.publish_time = src.publish_time;
  dest.summary = getConvertedNewsStr(src.summary);
  dest.tags = src.tags;
  dest.type = src.type;
  dest.title = getConvertedNewsStr(src.title);
  dest.topic_flag = src.topic_flag;
  dest.topics = src.topics;
  dest.hits = src.hits || 0;
  dest.layout = src.layout;
}

function getConvertedNewsStr(str) {
  if (str === undefined || str.length <= 0) return "";
  return replaceAngleBracket(str);
}

/**
 * 更新 新闻详情信息
 * @param src
 * @param dest
 */
function updateNewsDetail(src, dest) {
  if (src === undefined || dest === undefined) return;
  dest.content = parse_news_image(src.content);
  dest.create_time = src.create_time;
  dest.creator_id = src.creator_id;
  dest.creator_name = src.creator_name;
  dest.source = src.source;
  dest.update_time = src.update_time;
}

/**
 * 更新 新闻的互动信息
 * @param src
 * @param dest
 */
function updateNewsInteractions(src, dest) {
  if (dest.praise !== undefined) {
    src.praiseNum = dest.praise; // 点赞数量
  }
  if (dest.comment !== undefined) {
    src.commentsNum = dest.comment; // 评论数量
  }
  if (dest.is_praised !== undefined) {
    src.isPraise = dest.is_praised; // 是否已经点赞
  }
}

var mCategoryNewsReadMark = restoreNewsReadMark(); // 新闻已读标识数组

/**
 * 恢复 本地缓存的 新闻已读数据
 * @returns {*}
 */
function restoreNewsReadMark() {
  if (mSupportLocalStorage) {
    var restore = restoreLocalJsonData(KEY_STORE_CATEGORY_NEWS_READ_MARK);
    if (restore !== undefined) {
      return restore;
    }
  }
  return [];
}

/**
 * 本地缓存 新闻分类数据
 */
function storeNewsReadMark() {
  if (mSupportLocalStorage) {
    localStorage.setItem(KEY_STORE_CATEGORY_NEWS_READ_MARK, JSON.stringify(mCategoryNewsReadMark));
  }
}

/**
 * 标注新闻为 已读
 * @param news
 */
function markNewsHasRead(news) {
  if (news !== undefined) {
    news.isRead = true;
  }

  localStorage.setItem(READ_MARK + news.id, SUCCESS);
}

/**
 * 判断 新闻是否已读
 * @param news
 * @returns {*}
 */
function hasNewsRead(news) {
  if (news === undefined) return false;
  return localStorage.getItem(READ_MARK + news.id) == SUCCESS;
}

/**
 * 插入或更新 新闻
 * @param news
 * @returns {*}
 */
function insertOrUpdateCategoryNews(news) {

  if (news.id === undefined) return false;

  var found = findCategoryNews(news.id);
  var needToInsert = found === undefined;
  if (needToInsert) {
    mCategoryNews.push(buildNews(news));
    /*mCategoryNews.sort(function(a, b) {
      return a.publish_time < b.publish_time ? 1 : -1;
    });*/
  } else {
    updateNewsBase(news, found);
  }

  return needToInsert;
}

/**
 * 获取 指定分类中的 新闻
 * @param categoryID
 * @returns {*}
 */
function findCategoryNews(newID) {
  return findItem(mCategoryNews, newID);
}

/**
 * 本地缓存 直接存储category对象，category.newsArray可以获取该分类下的新闻
 */
function storeCategoryNews(category) {
  if (mSupportLocalStorage) {
    var key = category.id + '_' + newsKey;
    if (mPlatformAndroid) {
      JsSaveSp.removeItem(key);
      JsSaveSp.setItem(key, JSON.stringify(category));
    }
    if (mPlatformIOS) {
      localStorage.removeItem(key);
      localStorage.setItem(key, JSON.stringify(category));
    }
  }
}

/**
 * 恢复本地缓存 直接存储category对象，category.newsArray可以获取该分类下的新闻
 */
function restoreCategoryNews(category) {
  if (mSupportLocalStorage) {
    var data,
      key = category.id + '_' + newsKey;
    if (mPlatformAndroid) {
      data = JsSaveSp.getItem(key);
    }
    if (mPlatformIOS) {
      data = localStorage.getItem(key);
    }
    if (data) {
      return JSON.parse(data);
    }
  }
  return category;
}

/**
 * 根据新闻ID 删除新闻
 * @param newsID
 */
function deleteNews(news) {
  for (var i = 0, max = mCategoryNews.length; i < max; i++) {
    if (mCategoryNews[i].id === news.id) {
      mCategoryNews.splice(i, 1);
    }
  }

  if (news.categories !== undefined) {
    var categories = news.categories;
    for (var j = 0, cmax = categories.length; j < cmax; j++) {
      if (categories[j] !== undefined) {
        checkCategoryHasNews(categories[j]);
      }
    }
  }

  news = undefined;
}

/**
 * 判断 新闻是否属于 分类
 * @param news
 * @param categoryID
 * @returns {boolean}
 */
function isNewsBelongCategory(news, categoryID) {
  if (news === undefined) return false;

  var categories = news.categories;
  for (var i = 0, max = categories.length; i < max; i++) {
    if (categories[i] === categoryID) {
      return true;
    }
  }
  return false;
}

/**
 * 判断 是否是图文新闻
 * @param news
 * @returns {boolean}
 */
function isTypeOfNews(news) {
  if (news === undefined) return false;
  var type = news.type;
  return type === 0 || type === 1;
}

/**
 * 判断 是否是图集新闻
 * @param news
 * @returns {boolean}
 */
function isTypeOfGallery(news) {
  if (news === undefined) return false;
  var type = news.type;
  return type === 2;
}

/**
 * 判断 是否是专题
 * @param news
 * @returns {boolean}
 */
function isTypeOfTopic(news) {
  if (news === undefined) return false;
  return (news.topics !== undefined) && (news.topics.length >= 1);
}

/**
 * 判断 是否是专题 入口
 * @param news
 * @returns {boolean}
 */
function isTypeOfTopicEntrance(news) {
  if (news === undefined) return false;
  return news.topic_flag === 0;
}

/**
 * 获取新闻 热度
 * @param news
 * @returns {*}
 */
function getNewsHot(news) {
  if (news === undefined) return false;

  return news.hits;
}

/**
 * 获取 属于分类的新闻 数组对象
 * @param categoryID
 * @param allNews
 * @returns {Array}
 */
function getCategoryNewsArray(categoryID, allNews) {
  var result = [];

  if (categoryID !== undefined || allNews !== undefined) {
    for (var i = 0, max = allNews.length; i < max; i++) {
      var news = allNews[i];
      if (news !== undefined && isNewsBelongCategory(news, categoryID)) {
        // 新闻属于此分类
        if (isTypeOfTopic(news)) {
          // 属于此分类，并且是 专题
          if (isTypeOfTopicEntrance(news)) {
            // 专题入口，加入
            insertOrUpdate(result, news);
          }
        } else {
          // 非专题，加入
          insertOrUpdate(result, news);
        }
      }
    }
  }

  return result;
}


/**
 * 获取 指定的分类的新闻 最新新闻条目
 * @param categoryID
 * @param requestSize
 * @param callback
 */
function getCategoryNewsLatest(category, requestSize, callback) {
  if (category === undefined || category.id === undefined) {
    callback(category, FAILURE);
    return;
  }

  var maxts = 0;
  if (category.maxTs != INVALID) {
    maxts = category.maxTs;
  }

  $.getJSON("/news", {
    "category": category.id,
    "maxts": maxts,
    "size": requestSize
  }, function(data) {
    var inc = data.inc;
    var requestedNewsIDs = [];

    if (data.config) {
      HOT_NUM = +data.config.hthh;
    }

    data = data.items;

    mCategoryNews.length = 0;
    category.newsArray.length = 0;
    category.thumbNews.length = 0;
    category.minTs = INVALID;

    markCategoryNewsRequested(category);
    markCategoryHasOlderNews(category, true);
    markCategoryHasNews(category);

    if (data === undefined || data.length === 0) {
      category.hasNews = false;
      storeCategoryNews(category);
      callback(category, EMPTY);
      return;
    }

    for (var i = 0, max = data.length; i < max; i++) {
      var news = data[i];

      requestedNewsIDs.push(news.id);
      insertOrUpdateCategoryNews(news);
      updateCategoryTimemark(category, news);
    }

    if (isShowComments()) {
      getNewsInteractionsCounterBatch(requestedNewsIDs, function() {
        onEvent(EVENT_ELEMENT_REFRESH);
      });
    }

    mergeTopicNewsEntrance();

    category.appendSize = inc;
    category.newsArray = getCategoryNewsArray(category.id, mCategoryNews);
    //category.newsArray = category.newsArray.slice(0, category.newsArray.length);
    var count = 0;

    category.newsArray = category.newsArray.filter(function(item) {
      if (item.thumbnail && count++ < 3) {
        category.thumbNews.push(item);

        return false;
      }
      return true;
    });

    //dbcache缓存类别的新闻列表
    storeCategoryNews(category);

    callback(category, SUCCESS);

  }, function(data, statusCode) {
    var categoryNews = restoreCategoryNews(category);
    var requestedNewsIDs = [];

    markCategoryNewsRequested(category);

    markCategoryHasOlderNews(category, true);
    if (categoryNews === undefined || categoryNews.newsArray.length === 0) {
      callback(category);
      return;
    }
    category.newsArray = categoryNews.newsArray;
    category.thumbNews = categoryNews.thumbNews;

    categoryNews.newsArray.forEach(function(news) {
      if (news.hasOwnProperty('$$hashKey')) {
        delete news.$$hashKey;
      }

      requestedNewsIDs.push(news.id);
      insertOrUpdateCategoryNews(news);
    });

    if (isShowComments()) {
      getNewsInteractionsCounterBatch(requestedNewsIDs, function() {
        onEvent(EVENT_ELEMENT_REFRESH);
      });
    }

    if (categoryNews.newsArray.length > 0 || categoryNews.thumbNews.length > 0) {
      markCategoryHasNews(category);
    }

    mergeTopicNewsEntrance();

    if (!data.code || data.code ==='timeout') {
      callback(category, 600);
      return;
    }
    callback(category);
  });

}

/**
 * 分页请求 新闻
 * @param categoryID
 * @param requestSize
 * @param callback
 */
function getCategoryNewsOlder(category, requestSize, callback) {
  var size = 0;
  if (category === undefined || category.id === undefined) {
    callback(category, FAILURE, size);
    return;
  }

  $.getJSON("/news", {
    "category": category.id,
    "mints": category.minTs !== undefined ? category.minTs : 0,
    "size": requestSize
  }, function(data) {
    data = data.items.filter(function(item) {
      return !item.icons.length;
    });
    markCategoryNewsRequested(category);

    if (data === undefined || data.length === 0) {
      markCategoryHasOlderNews(category, false);
      callback(category, EMPTY, 0);
      return;
    }

    if (category.thumbNews && category.thumbNews.length > 0) {
      var thumbNewsIds = category.thumbNews.map(function(news) {
        return news.id;
      });
      var n = data.length;

      while (n--) {
        if (thumbNewsIds.indexOf(data[n].id) !== -1) {
          data.splice(n, 1);
        }
      }
    }

    if (data.length === 0) {
      markCategoryHasOlderNews(category, false);
      callback(category, EMPTY, 0);
      return;
    }

    var requestedNewsIDs = [];
    for (var i = 0, max = data.length; i < max; i++) {
      var news = data[i];
      requestedNewsIDs.push(news.id);

      insertOrUpdateCategoryNews(news);
      updateCategoryTimemark(category, news);
    }

    if (isShowComments()) {
      getNewsInteractionsCounterBatch(requestedNewsIDs, function() {
        onEvent(EVENT_ELEMENT_REFRESH);
      });
    }

    mergeTopicNewsEntrance();

    category.newsArray.length = 0;
    category.newsArray = getCategoryNewsArray(category.id, mCategoryNews);
    category.newsArray = category.newsArray.slice(0, category.newsArray.length);

    if (category.thumbNews && category.thumbNews.length > 0) {
      var thumbNewsIds1 = category.thumbNews.map(function(news) {
        return news.id;
      });
      var n1 = category.newsArray.length;

      while (n1--) {
        if (thumbNewsIds1.indexOf(category.newsArray[n1].id) !== -1) {
          category.newsArray.splice(n1, 1);
        }
      }
    }

    callback(category, SUCCESS, data.length);

  }, function(data, statusCode) {
    // 无网络，标记无较早的新闻
    markCategoryHasOlderNews(category, false);

    callback(category, FAILURE, size);
  });
}

/*------------------------------------------------------------------------------*/

// 当前新闻详情ID，包括（图文新闻 和 图集新闻）
var mCurrentDetailNewsID; // 当前 详情新闻 ID
var mCurrentDetailNews; // 当前 详情新闻 对象

var mCurrentDetailNewsCategoryID; // 当前 详情新闻 所属分类ID
var mCurrentSameNews = []; // 当前 详情新闻 同类新闻

var mCurrentDetailNewsIndexInSimilar = 0; // 当前 详情新闻 在同类所有新闻列表中的位置

var mPreviousDetailNews; // 当前 详情新闻 所属分类 同类新闻 上一条新闻
var mNextDetailNews; // 当前 详情新闻 所属分类 同类新闻 下一条新闻

/**
 * 设置 当前新闻详情ID
 * @param newsID
 */
function setCurrentDetailNewsID(newsID) {
  if (newsID === undefined) return;

  mCurrentDetailNewsID = newsID;
  if (mSupportLocalStorage) {
    localStorage.setItem(KEY_STORE_CURRENT_DETAIL_NEWS_ID, newsID);
  }
}

/**
 * 获取 当前新闻详情ID
 * @returns {*}
 */
function getCurrentDetailNewsID() {
  if (mCurrentDetailNewsID !== undefined) return mCurrentDetailNewsID;

  if (mSupportLocalStorage) {
    mCurrentDetailNewsID = localStorage.getItem(KEY_STORE_CURRENT_DETAIL_NEWS_ID);
  }

  return mCurrentDetailNewsID;
}

/**
 * 设置 当前 详情新闻 的分类ID
 * @param categoryID
 */
function setCurrentDetailNewsCategoryID(categoryID) {
  if (categoryID === undefined) return;

  mCurrentDetailNewsCategoryID = categoryID;
  if (mSupportLocalStorage) {
    localStorage.setItem(KEY_STORE_CURRENT_DETAIL_NEWS_CATEGORY_ID, categoryID);
  }
}

/**
 * 获取 当前 详情新闻 的分类ID
 * @returns {undefined}
 */
function getCurrentDetailNewsCategoryID() {
  if (mCurrentDetailNewsCategoryID !== undefined) return mCurrentDetailNewsCategoryID;

  if (mSupportLocalStorage) {
    mCurrentDetailNewsCategoryID = localStorage.getItem(KEY_STORE_CURRENT_DETAIL_NEWS_CATEGORY_ID);
  }

  return mCurrentDetailNewsCategoryID;
}

/**
 * 新闻 按发布时间 降序排序
 * @param newsArray
 */
function sortNewsByPublishTime(newsArray) {
  if (newsArray.length > 0) {
    newsArray.sort(function(m, n) {
      return m.publish_time - n.publish_time;
    }).reverse();
  }
}

/**
 * 加载 相同的所有新闻
 * @param news
 */
// function loadSimilarNews(loadFromServer, news, callback) {
//   if (!loadFromServer) {
//     if (isTypeOfTopic(news)) {
//       loadSameTopicNews(getCurrentTopicID(), news.id);
//     } else {
//       loadSameCategoryNews(getCurrentDetailNewsCategoryID(), news.id);
//     }
//     callback();

//   } else {
//     var timestamp = +news.publish_time;
//     var requestSize = 10;

//     if (isTypeOfTopic(news)) {
//       var topicID = getCurrentTopicID();
//       if (mCurrentDetailTopic === undefined) {
//         mCurrentDetailTopic = new Topic(topicID);
//         updateTopicTimemark(timestamp);
//       }
//       loadSameTopicNewsByTimestamp(topicID, timestamp, requestSize, function(statusCode) {
//         loadSameTopicNews(topicID, news.id);
//         callback();
//       });
//     } else {
//       var categoryID = getCurrentCategoryID();
//       if (mCurrentCategory === undefined) {
//         mCurrentCategory = new Category(categoryID);
//         updateCategoryTimemark(mCurrentCategory, timestamp);
//         insertOrUpdateCategory(mCurrentCategory);
//       }

//       loadSameCategoryNewsByTimestamp(categoryID, timestamp, requestSize, function(statusCode) {
//         loadSameCategoryNews(categoryID, news.id);
//         callback();
//       });
//     }

//   }

// }

/**
 * 按照分类 加载相同的所有新闻
 * @param categoryID
 */
function loadSameCategoryNews(categoryID, newsID) {
  mCurrentSameNews.length = 0;
  mPreviousDetailNews = undefined;
  mNextDetailNews = undefined;

  var i = 0;
  var news;
  for (; i < mCategoryNews.length; i++) {
    news = mCategoryNews[i];
    // FIXME 暂时只加载 图文新闻
    if (news !== undefined && isTypeOfNews(news) && !isTypeOfTopicEntrance(news) && isNewsBelongCategory(news, categoryID)) {
      mCurrentSameNews.push(news);
    }
  }

  sortNewsByPublishTime(mCurrentSameNews);

  for (i = 0; i < mCurrentSameNews.length; i++) {
    news = mCurrentSameNews[i];
    if (news.id === newsID) {

      mCurrentDetailNewsIndexInSimilar = i;

      if (i > 0) {
        mPreviousDetailNews = mCurrentSameNews[i - 1];
      }
      if (i + 1 < mCurrentSameNews.length) {
        mNextDetailNews = mCurrentSameNews[i + 1];
      }
    }
  }
}

function loadSameCategoryNewsByTimestamp(categoryId, timestamp, requestSize, callback) {
  if (categoryId === undefined || timestamp === undefined) {
    callback(FAILURE);
    return;
  }

  $.getJSON("/categories/" + categoryId + "/nearbynews?time=" + timestamp + "&size=" + requestSize, {},
    function(data) {

      if (data === undefined) {
        callback(EMPTY);
        return;
      }

      var requestedNewsIDs = [],
        news;
      for (var i = 0, max = data.before.length; i < max; i++) {
        news = data.before[i];
        requestedNewsIDs.push(news.id);

        insertOrUpdateCategoryNews(news);
        updateCategoryTimemark(mCurrentCategory, news);
      }
      for (var j = 0, amax = data.after.length; j < amax; j++) {
        news = data.after[j];
        requestedNewsIDs.push(news.id);

        insertOrUpdateCategoryNews(news);
        updateCategoryTimemark(mCurrentCategory, news);
      }

      getNewsInteractionsCounterBatch(requestedNewsIDs, function() {
        onEvent(EVENT_ELEMENT_REFRESH);
      });

      mergeTopicNewsEntrance();

      storeCategories();

      callback(SUCCESS);

    },
    function(data, statusCode) {
      callback(data, FAILURE);
    });
}

/**
 * 按照专题 加载相同的所有新闻
 * @param categoryID
 */
function loadSameTopicNews(topicID, newsID) {
  mCurrentSameNews.length = 0;
  mPreviousDetailNews = undefined;
  mNextDetailNews = undefined;

  var i = 0;
  var news;
  for (; i < mCategoryNews.length; i++) {
    news = mCategoryNews[i];
    // FIXME 暂时只加载 图文新闻
    if (news !== undefined && isTypeOfNews(news) && isNewsBelongTopic(news, topicID)) {
      mCurrentSameNews.push(news);
    }
  }

  sortNewsByPublishTime(mCurrentSameNews);

  for (i = 0; i < mCurrentSameNews.length; i++) {
    news = mCurrentSameNews[i];
    if (news.id === newsID) {

      mCurrentDetailNewsIndexInSimilar = i;

      if (i > 0) {
        mPreviousDetailNews = mCurrentSameNews[i - 1];
      }
      if (i + 1 < mCurrentSameNews.length) {
        mNextDetailNews = mCurrentSameNews[i + 1];
      }
    }
  }
}

function loadSameTopicNewsByTimestamp(topicID, timestamp, requestSize, callback) {
  if (topicID === undefined || timestamp === undefined) {
    callback(FAILURE);
    return;
  }

  $.getJSON("/topics/" + topicID + "/nearbynews?time=" + timestamp + "&size=" + requestSize, {},
    function(data) {

      markTopicNewsRequested();

      if (data === undefined) {
        callback(EMPTY);
        return;
      }

      var requestedNewsIDs = [],
        news;
      for (var i = 0, bmax = data.before.length; i < bmax; i++) {
        news = data.before[i];
        requestedNewsIDs.push(news.id);

        insertOrUpdateCategoryNews(news);
        updateTopicTimemark(news.publish_time);
      }

      for (var j = 0, amax = data.after.length; j < amax; j++) {
        news = data.after[j];
        requestedNewsIDs.push(news.id);

        insertOrUpdateCategoryNews(news);
        updateTopicTimemark(news.publish_time);
      }

      getNewsInteractionsCounterBatch(requestedNewsIDs, function() {
        onEvent(EVENT_ELEMENT_REFRESH);
      });

      mergeTopicNewsEntrance();

      callback(SUCCESS);

    },
    function(data, statusCode) {
      callback(data, FAILURE);
    });
}

/**
 * 判断 用户进行新闻翻页时 是否需要请求较早的新闻
 * @param categoryID
 * @returns {*}
 */
function loadOlderNewsWhenFlipIfNeed(news) {
  var offset = 2;

  if (mCurrentSameNews !== undefined &&
    mCurrentSameNews.length > 0) {
    var remain = mCurrentSameNews.length - mCurrentDetailNewsIndexInSimilar;

    if (isTypeOfTopic(news)) {
      if (mCurrentDetailTopic !== undefined && isCurrentTopicHasOlderNews() && (remain <= offset)) {
        loadSameTopicNewsByTimestamp(getCurrentTopicID(), news.publish_time, DEFAULT_REQUEST_SIZE, function(statusCode) {
          onEvent(EVENT_ELEMENT_REFRESH);
        });
      }

    } else {
      var category = mCurrentCategory;
      if (category !== undefined && category.moreOlderNews && (remain <= offset)) { // 翻到 倒数第2条时 允许请求
        loadSameCategoryNewsByTimestamp(category.id, news.publish_time, DEFAULT_REQUEST_SIZE, function(statusCode) {
          onEvent(EVENT_ELEMENT_REFRESH);
        });
      }
    }
  }
}

/**
 * 判断 是否有 上一条新闻
 * @returns {boolean}
 */
function hasPreviousNews() {
  return mPreviousDetailNews !== undefined;
}

/**
 * 判断 是否有 下一条新闻
 * @returns {boolean}
 */
function hasNextNews() {
  return mNextDetailNews !== undefined;
}

/**
 * 获取 上一条新闻的ID
 * @returns {*}
 */
function getPreviousNewsID() {
  if (mPreviousDetailNews === undefined) return undefined;
  return mPreviousDetailNews.id;
}

/**
 * 获取 下一条新闻的ID
 * @returns {*}
 */
function getNextNewsID() {
  if (mNextDetailNews === undefined) return undefined;
  return mNextDetailNews.id;
}

/**
 * 获取 上一条新闻的标题
 * @returns {*}
 */
function getPreviousNewsTitle() {
  if (mPreviousDetailNews === undefined) return "news_detail_normal_no_news";
  return mPreviousDetailNews.title;
}

/**
 * 获取 上一条新闻的标题色值
 * @returns {*}
 */
function getPreviousNewsTitleColor() {
  if (mPreviousDetailNews === undefined) return '#808080';
  return '#fc7587';
}

/**
 * 获取 下一条新闻的标题
 * @returns {*}
 */
function getNextNewsTitle() {
  if (mNextDetailNews === undefined) return "news_detail_normal_no_news";
  return mNextDetailNews.title;
}
/**
 * 获取 下一条新闻的标题色值
 * @returns {*}
 */
function getNextNewsTitleColor() {
  if (mNextDetailNews === undefined) return '#808080';
  return '#fc7587';
}

/**
 * 判断 是否要显示 新闻翻页按钮
 * @returns {boolean}
 */
function showFlipButtonIfNeed() {
  return true;
}

/**
 * 获取 新闻详情
 * @param id
 */
function getNewsDetail(newsID, callback) {

  var news = findCategoryNews(newsID);
  if (news === undefined) {
    news = new News(newsID);
  }

  // 标注 新闻为已读
  markNewsHasRead(news);

  //storeCategoryNews();
  $.getJSON("/news/" + newsID, {}, function(data) {

    //准备收藏新闻
    try {
      if (mPlatformAndroid) {
        JsHttpRequest.getNewsDetailSuccess(JSON.stringify(data));
      } else if (mPlatformIOS) {
        MUICordovaWebJSExport.getNewsDetailSuccess(JSON.stringify(data));
      }
    } catch(e){
      log('准备收藏新闻失败');
    }

    //mCurrentDetailNews = new News( newsID );
    //updateNewsBase( news, mCurrentDetailNews );
    //updateNewsDetail( data, mCurrentDetailNews );

    // 通过新闻详情中的数据，更新新闻基本信息
    updateNewsBase(data, news);
    mCurrentDetailNews = news;
    updateNewsDetail(data, mCurrentDetailNews);

    //var loadSimilarFromServer = mCategoryNews.length <= 0;

    insertOrUpdateCategoryNews(news);

    /*loadSimilarNews(loadSimilarFromServer, news, function() {
      loadOlderNewsWhenFlipIfNeed(news);
      onEvent(EVENT_ELEMENT_REFRESH);
    });*/

    // 获取互动计数
    getNewsInteractionsCounter(newsID, function() {
      if (news !== undefined) {
        mCurrentDetailNews.praiseNum = news.praiseNum;
        mCurrentDetailNews.commentsNum = news.commentsNum;
        mCurrentDetailNews.isPraise = news.isPraise;
      }
    });

    callback(SUCCESS);
  }, function(data, statusCode) {

    if (data !== undefined) {
      if (ERROR_CODE_NEWS_DATA_NOT_EXIST === data.code) {
        // 新闻对象被删除
        deleteNews(news);
        callback(DELETED);
        return;
      }
    }

    if (data.code === 'NEWS/OUT_OF_SERVICE') {
      callback(402);
      return;
    }

    if (data.code === 'NEWS/SERVICE_NOT_FOUND') {
      callback(404);
      return;
    }

    callback(FAILURE);
  });
}


/**
 * 更新 新闻的互动数目（点赞、评论）
 * @param newsID
 * @param callback
 */
function getNewsInteractionsCounter(newsID, callback) {
  if (!isShowComments()) {
    return;
  }

  if (newsID === undefined) {
    callback(EMPTY);
    return;
  }

  $.getJSON(HOST_SDP_INTERACTIONS + "counters/object/NEWS/OBJECT/" + newsID, {},
    function(data) {
      if (data !== undefined) {
        var news = findCategoryNews(newsID);
        if (news !== undefined) {
          updateNewsInteractions(news, data);
        }
      }

      callback(SUCCESS);
    },
    function(data, statusCode) {
      callback(FAILURE);
    });
}

/**
 * 批量更新 新闻的互动数目（点赞、评论）
 * @param newsIDs
 * @param callback
 */
function getNewsInteractionsCounterBatch(newsIDs, callback) {
  if (newsIDs === undefined || newsIDs.length <= 0) {
    callback(EMPTY);
    return;
  }

  $.postJSON(HOST_SDP_INTERACTIONS + "counters/object/NEWS/OBJECT", {
      "object_ids": newsIDs
    },
    function(data) {

      if (data !== undefined && data.items !== undefined) {
        for (var i = 0; i < data.items.length; i++) {
          var counter = data.items[i];
          var news = findCategoryNews(counter.id);
          if (news !== undefined) {
            updateNewsInteractions(news, counter);
          }
        }
        callback(SUCCESS);
      } else {
        callback(EMPTY);
      }
    },
    function(data, statusCode) {
      callback(FAILURE);
    });
}

/**
 * 批量更新 新闻的互动数目（点赞、评论）自动分组请求
 * @param newsIDs
 * @param callback
 */
function getNewsInteractionsCounterBatchWithAutoAverage(newsIDs, callback) {
  if (newsIDs === undefined || newsIDs.length <= 0) {
    callback(EMPTY);
    return;
  }

  var request_queue_size = 100;
  var repeatCount = Math.round(newsIDs.length / request_queue_size) + 1;
  for (var i = 0; i < repeatCount; i++) {
    var queue = newsIDs.splice(0, request_queue_size);
    getNewsInteractionsCounterBatch(queue, callback);
  }
}

/**
 * 按照 新闻分类 批量更新 新闻的互动数目（点赞、评论）自动分组请求
 * @param categoryID
 */
function updateNewsInteractionsCounterByCategory(category, callback) {
  if (category !== undefined) {
    var categoryNews = category.newsArray;
    var newsIDs = [];
    for (var i = 0; i < categoryNews.length; i++) {
      if (categoryNews[i] !== undefined) {
        newsIDs.push(categoryNews[i].id);
      }
    }
    getNewsInteractionsCounterBatchWithAutoAverage(newsIDs, callback);
  }


}

/**
 * 按照 新闻专题 批量更新 新闻的互动数目（点赞、评论）自动分组请求
 * @param topicID
 */
function updateNewsInteractionsCounterByTopic(topicID, callback) {
  var topicNews = getTopicNewsArray(topicID, mCategoryNews);
  var newsIDs = [];
  for (var i = 0, max = topicNews.length; i < max; i++) {
    if (topicNews[i] !== undefined) {
      newsIDs.push(topicNews[i].id);
    }
  }
  getNewsInteractionsCounterBatchWithAutoAverage(newsIDs, callback);
}

/**
 * 对新闻 点赞或者取消点赞
 * @param newsID
 * @param isPraise
 */
function markNewsIsPraise(newsID, isPraise) {
  var news = findCategoryNews(newsID);
  if (news !== undefined) {
    news.isPraise = isPraise;
  }
}

/**
 * 点赞新闻
 * @param newsID
 * @param newsAuthorUid
 * @param callback
 */
function praiseNews(newsID, callback) {
  $.postJSON(HOST_SDP_INTERACTIONS + "interactions", {
    "irt_type": "PRAISE",
    "biz_type": "NEWS",
    "object_type": "OBJECT",
    "object_id": newsID
  }, function(data) {
    callback(SUCCESS);
  }, function(data, statusCode) {
    if (statusCode === 400) {
      // 新闻已经点赞
      markNewsIsPraise(newsID, false);
    }
    callback(FAILURE);
  });
}

/**
 * 取消点赞新闻
 * @param newsID
 * @param callback
 */
function cancelPraiseNews(newsID, callback) {
  $.deleteJSON(HOST_SDP_INTERACTIONS + "interactions/PRAISE/NEWS/OBJECT/" + newsID, {},
    function(data) {
      callback(SUCCESS);
    },
    function(data, statusCode) {
      if (statusCode === 400) {
        markNewsIsPraise(newsID, true);
      }

      callback(FAILURE);
    });
}

/**
 * 重置 新闻评论相关配置信息
 */
function commentReset() {
  cleanComments();

  if (mCurrentCommentConfig !== undefined) {
    mCurrentCommentConfig.max_cmt_id = INVALID;
    mCurrentCommentConfig.min_cmt_id = INVALID;
    mCurrentCommentConfig.requestTime = undefined;
    mCurrentCommentConfig.hasOlderComment = true;
  }
}

var mCurrentComments = [];
var mCurrentCommentConfig = new CommentConfig();

function buildComment(data) {
  var comment = new Comment(data.id);
  updateComment(data, comment);
  return comment;
}

/**
 * 更新 新闻评论
 * @param src
 * @param dest
 */
function updateComment(src, dest) {
  if (src === undefined || dest === undefined) return;
  dest.cmt_id = src.cmt_id;
  dest.news_id = src.object_id;
  dest.content = getFilteredCommentContent(src.content);
  dest.ower_uid = src.uid;
  if (src.op_time) {
    dest.time = getCommentFormatedISO8601Time(src.op_time);
  } else {
    dest.time = "";
  }
}

/**
 * 获取 过滤后的 评论内容
 * @param content
 * @returns {*}
 */
function getFilteredCommentContent(content) {
  var replacedStr = content.replaceAll('\n', '<br>'); // 替换 换行
  replacedStr = replacedStr.replaceAll(' ', '&nbsp;'); // 替换 空格
  replacedStr = replaceURLWithHTMLLinks(replacedStr);
  return replacedStr;
}

/**
 * 获取 格式化后的 评论时间
 * @param time
 * @returns {*}
 */
function getCommentFormatedISO8601Time(time) {
  var publishDate = new Date(convertISO8601toDate(time));
  var currentDate = new Date();
  var interval = currentDate.getTime() - publishDate.getTime();

  if (interval > 1000 * 60 * 60 * 24 || interval < 0) {
    return publishDate.format(DEFAULT_TIME_FORMAT);
  }

  var h = parseInt(interval / (1000 * 60 * 60));
  if (0 === h) {
    var m = parseInt(interval / (1000 * 60));
    if (0 === m) {
      return window.translate("just_now");
    }
    return m + window.translate("minute_ago");
  }
  return h + window.translate("hour_ago");
}


/**
 * 插入或更新 新闻评论
 * @param news
 * @returns {*}
 */
function insertOrUpdateComment(comment) {

  if (comment.id === undefined) return;

  var found = findComment(comment.id);
  if (found === undefined) {
    mCurrentComments.push(buildComment(comment));
  } else {
    updateComment(comment, found);
  }

}

/**
 * 获取 指定的 新闻的评论
 * @param categoryID
 * @returns {*}
 */
function findComment(commentID) {
  return findItem(mCurrentComments, commentID);
}

/**
 * 清空当前的新闻评论
 */
function cleanComments() {
  mCurrentComments.length = 0;
}

/**
 * 判断 当前是否有评论
 * @returns {boolean}
 */
function hasComments() {
  return mCurrentComments.length > 0;
}

/**
 * 刷新 评论的更新时间戳
 * @param time
 */
function updateCommentCmtMark(cmtID) {
  if (mCurrentCommentConfig === undefined) return;
  // min
  if (INVALID === mCurrentCommentConfig.min_cmt_id || cmtID < mCurrentCommentConfig.min_cmt_id) {
    mCurrentCommentConfig.min_cmt_id = cmtID;
  }

  // max
  if (INVALID === mCurrentCommentConfig.max_cmt_id || cmtID > mCurrentCommentConfig.max_cmt_id) {
    mCurrentCommentConfig.max_cmt_id = cmtID;
  }
}

/**
 * 获取 最近一次评论刷新时间
 * @returns {*}
 */
function getCommentRequestTime() {
  if (mCurrentCommentConfig === undefined ||
    mCurrentCommentConfig.requestTime === undefined) {
    return undefined;
  } else {
    return new Date(mCurrentCommentConfig.requestTime).format(DEFAULT_TIME_FORMAT);
  }
}

/**
 * 更新 最近一次评论刷新时间
 */
function updateCommentRequestTime() {
  if (mCurrentCommentConfig !== undefined) {
    mCurrentCommentConfig.requestTime = getNowUnixTime();
  }
}

/**
 * 判断 是否还有旧的评论需要获取
 */
function hasOlderComment() {
  if (mCurrentCommentConfig !== undefined) {
    return mCurrentCommentConfig.hasOlderComment;
  }
  return false;
}

/**
 * 标记 是否还有旧的评论需要获取
 * @param has
 */
function markHasOlderComment(has) {
  if (mCurrentCommentConfig !== undefined) {
    mCurrentCommentConfig.hasOlderComment = has;
  }
}

/**
 * 获取 评论中的用户名
 * @param comment
 * @returns {*}
 */
function getCommentUserName(comment) {
  if (comment) {
    return getUserNameFromCache(comment.ower_uid);
  }
  return UNKNOWN_USER_NAME;
}

/**
 * 更新 评论中的用户信息
 * @param comments
 */
function updateCommentUserNames(comments) {
  if (comments === undefined || comments.items === undefined) return;

  var names = [];
  for (var i = 0, max = comments.items.length; i < max; i++) {
    var uid = comments.items[i].uid;
    if (uid !== undefined && isUserInfoOutOfDate(uid)) {
      names.push(uid);
    }
  }
  if (names.length > 0) {
    updateCommentUserNameBatch(names, function(code) {
      onEvent(EVENT_ELEMENT_REFRESH);
    });
  }
}


/**
 * 获取 最新的新闻评论
 * @param newsID
 * @param size
 * @param callback
 */
function getNewsCommentsLatest(newsID, size, callback) {
  if (newsID === undefined) {
    callback(FAILURE);
    return;
  }

  var min_cmt_id = mCurrentCommentConfig.max_cmt_id != INVALID ? mCurrentCommentConfig.max_cmt_id : 0; // 返回cmt_id大于min_cmt_id的评论
  $.getJSON(HOST_SDP_INTERACTIONS + "comments/list/toobj/NEWS/" + newsID, {
    "min_cmt_id": 0,
    "$limit": size,
    "$count": "true"
  }, function(data) {

    cleanComments();
    mCurrentCommentConfig.min_cmt_id = INVALID;
    markHasOlderComment(true);

    if (data.items) {

      for (var i = 0, max = data.items.length; i < max; i++) {
        var comment = data.items[i];
        insertOrUpdateComment(comment);
        updateCommentCmtMark(comment.cmt_id);
      }

      updateCommentUserNames(data);
      updateCommentRequestTime();

      sortCommentByCmtID();

      callback(SUCCESS);
    } else {
      callback(EMPTY);
    }

  }, function(data, statusCode) {
    callback(FAILURE);
  });
}

/**
 * 获取 比较旧的新闻评论
 * @param newsID
 * @param size
 * @param callback
 */
function getNewsCommentsOlder(newsID, size, callback) {
  if (newsID === undefined) {
    callback(FAILURE);
    return;
  }

  if (mCurrentCommentConfig !== undefined &&
    mCurrentCommentConfig.min_cmt_id !== undefined) {

    $.getJSON(HOST_SDP_INTERACTIONS + "comments/list/toobj/NEWS/" + newsID, {
      "max_cmt_id": mCurrentCommentConfig.min_cmt_id, // 返回cmt_id小于max_cmt_id的评论，默认为正无穷
      "$limit": size,
      "$count": "true"
    }, function(data) {
      if (data !== undefined && data.items !== undefined && data.items.length > 0) {
        // 有数据，表示有旧评论
        markHasOlderComment(true);

        for (var i = 0, max = data.items.length; i < max; i++) {
          var comment = data.items[i];
          insertOrUpdateComment(comment);
          updateCommentCmtMark(comment.cmt_id);
        }

        updateCommentUserNames(data);
        updateCommentRequestTime();

        sortCommentByCmtID();

        callback(SUCCESS);
      } else {
        // 无数据，表示无旧评论
        markHasOlderComment(false);
        callback(EMPTY);
      }

    }, function(data, statusCode) {
      callback(FAILURE);
    });
  } else {
    callback(FAILURE);
  }

}

/**
 * 评论 按发布CmtID 降序排序
 * @param newsArray
 */
function sortCommentByCmtID() {
  if (mCurrentComments !== undefined && mCurrentComments.length > 0) {
    mCurrentComments.sort(function(m, n) {
      return m.cmt_id - n.cmt_id;
    }).reverse();
  }
}

/**
 * 发布 新闻评论
 * @param newsID
 * @param content
 * @param callback
 */
function postComment(newsID, content, callback) {
  if (newsID === undefined) {
    callback(FAILURE);
    return;
  }

  $.postJSON(HOST_SDP_INTERACTIONS + "comments", {
      "biz_type": "NEWS",
      "object_type": "OBJECT",
      "object_id": newsID,
      "content": content
    },
    function(data) {
      log("postComment !!!");
      log(data);

      callback(SUCCESS);
    },
    function(data, statusCode) {
      callback(FAILURE);
      log("postComment :: error :: " + statusCode + " :: " + data);
    });
}


function buildPraiser(data) {
  return new Praiser(data.uid);
}

function updatePraiser(src, dest) {
  dest.irt_id = src.irt_id;
  dest.irt_type = src.irt_type;
  dest.op_time = src.op_time;
}

var mCurrentPraisers = [];

function getCurrentPraisers() {
  return mCurrentPraisers;
}

function hasPraisers() {
  return mCurrentPraisers && mCurrentPraisers.length > 0;
}

function cleanCurrentPraisers() {
  mCurrentPraisers.length = 0;
}

function insertOrUpdatePraiser(data) {
  if (data === undefined || data.uid === undefined) return;

  for (var i = 0, max = mCurrentPraisers.length; i < max; i++) {
    if (mCurrentPraisers[i].uid === data.uid) {
      updatePraiser(data, mCurrentPraisers[i]);
      return;
    }
  }
  var newPraiser = buildPraiser(data);
  updatePraiser(data, newPraiser);
  mCurrentPraisers.push(newPraiser);
}

function getPraiserName(praiser) {
  if (praiser) {
    return getUserNameFromCache(praiser.uid);
  }
  return UNKNOWN_USER_NAME;
}

/**
 * 获取新闻的点赞用户详情
 * @param newsID
 * @param requestSize
 * @param callback
 */
function getNewsPraisersLatest(newsID, requestSize, callback) {
  if (newsID === undefined) {
    callback(FAILURE);
    return;
  }

  $.getJSON(HOST_SDP_INTERACTIONS + "interactions/object_irt_list/PRAISE/NEWS/OBJECT/" + newsID +
    "?size=" + requestSize, {},
    function(data) {
      cleanCurrentPraisers();

      if (data === undefined || data.items === undefined || data.items.length <= 0) {
        callback(EMPTY);
        return;
      }

      var uids = [];
      for (var i = 0, max = data.items.length; i < max; i++) {
        var item = data.items[i];
        insertOrUpdatePraiser(item);
        uids.push(item.uid);
      }

      updateCommentUserNameBatch(uids, function() {
        onEvent(EVENT_ELEMENT_REFRESH);
      });

      callback(SUCCESS);
    },
    function(data, statusCode) {
      callback(FAILURE);
    });
}

function buildTopic(data) {
  if (data === undefined) return undefined;
  var topic = new Topic(data.id);
  topic.create_time = data.create_time;
  topic.name = data.name;
  topic.summary = data.summary;
  topic.update_time = data.update_time;
  topic.cover_img = data.cover_img;
  return topic;
}

var mCurrentDetailTopicID; // 当前 专题详情 ID
var mCurrentDetailTopic; // 当前 专题详情 对象

/**
 * 设置 当前 专题详情ID
 * @param topicID
 */
function setCurrentTopicID(topicID) {
  if (topicID === undefined) return;

  mCurrentDetailTopicID = topicID;
  if (mSupportLocalStorage) {
    localStorage.setItem(KEY_STORE_CURRENT_TOPIC_ID, topicID);
  }
}

/**
 * 获取 当前 专题详情ID
 * @returns {*}
 */
function getCurrentTopicID() {
  if (mCurrentDetailTopicID !== undefined) return mCurrentDetailTopicID;

  if (mSupportLocalStorage) {
    mCurrentDetailTopicID = localStorage.getItem(KEY_STORE_CURRENT_TOPIC_ID);
  }

  return mCurrentDetailTopicID;
}

function getCurrentTopic() {
  return mCurrentDetailTopic;
}

/**
 * 获取 专题新闻 上次请求的时间
 * @returns {*}
 */
function getTopicNewsLastUpdateTime() {
  if (mCurrentDetailTopic !== undefined && mCurrentDetailTopic.lastUpdateNewsTime > 0) {
    return new Date(mCurrentDetailTopic.lastUpdateNewsTime).format(DEFAULT_TIME_FORMAT);
  }
  return undefined;
}

/**
 * 标注 专题新闻 是否被获取过
 */
function markTopicNewsRequested() {
  if (mCurrentDetailTopic !== undefined) {
    mCurrentDetailTopic.lastUpdateNewsTime = Math.max(mCurrentDetailTopic.maxTs, getNowUnixTime());
    mCurrentDetailTopic.newsRequested = true;
  }
}

/**
 * 标注 专题新闻 是否有较旧的新闻需要获取
 * @param has
 */
function markTopicHasOlderNews(has) {
  if (mCurrentDetailTopic !== undefined) {
    mCurrentDetailTopic.moreOlderNews = has;
  }
}

/**
 * 判断 当前专题 是否有 较早的新闻需要加载
 * @returns {*}
 */
function isCurrentTopicHasOlderNews() {
  if (mCurrentDetailTopic === undefined) return false;
  return mCurrentDetailTopic.moreOlderNews && mCurrentDetailTopic.minTs != INVALID;
}

/**
 * 判断 当前专题 是否有 新闻
 * @returns {boolean}
 */
function isCurrentTopicHasNews() {
  return mCurrentDetailTopic.newsArray.length > 0;
}

/**
 * 获取 专题详情
 * */
function getTopicDetail(topicID, callbackTopic) {
  $.getJSON("/topics/" + topicID, {}, function(data) {
    mCurrentDetailTopic = buildTopic(data);
    callbackTopic(SUCCESS);
  }, function(data, statusCode) {
    callbackTopic(FAILURE);
  });
}

/**
 * 判断 新闻是否属于 分类
 * @param news
 * @param categoryID
 * @returns {boolean}
 */
function isNewsBelongTopic(news, topicID) {
  if (news === undefined) return false;

  var topics = news.topics;
  for (var i = 0, max = topics.length; i < max; i++) {
    if (topics[i] === topicID) {
      return true;
    }
  }
  return false;
}

/**
 * 获取 属于专题的新闻 数组对象
 * @param topicID
 * @param allNews
 * @returns {Array}
 */
function getTopicNewsArray(topicID, allNews) {
  var result = [];
  if (allNews !== undefined && allNews.length > 0) {
    for (var i = 0, max = allNews.length; i < max; i++) {
      var news = allNews[i];
      if (isNewsBelongTopic(news, topicID)) {
        result.push(news);
      }
    }
  }
  return result;
}

/**
 * 刷新 分类新闻的更新时间戳
 * @param categoryID
 * @param time
 */
function updateTopicTimemark(time) {
  if (mCurrentDetailTopic !== undefined) {
    // min
    if (INVALID === mCurrentDetailTopic.minTs || time < mCurrentDetailTopic.minTs) {
      mCurrentDetailTopic.minTs = time;
    }

    // max
    if (INVALID === mCurrentDetailTopic.maxTs || time > mCurrentDetailTopic.maxTs) {
      mCurrentDetailTopic.maxTs = time;
    }
  }
}

/**
 * 获取 最新的 专题新闻
 * @param categoryID
 * @param topicID
 * @param requestSize
 * @param callback
 */
function getTopicNewsLatest(topicID, requestSize, callback) {

  var maxTs = (mCurrentDetailTopic !== undefined && mCurrentDetailTopic.maxTs != INVALID) ? mCurrentDetailTopic.maxTs : 0;

  $.getJSON("/news", {
    "topic": topicID,
    "maxts": maxTs,
    "size": requestSize
  }, function(data) {
    if (data.config) {
      HOT_NUM = +data.config.hthh;
    }
    data = data.items;

    markTopicNewsRequested();

    if (data === undefined || data.length <= 0) {
      markTopicHasOlderNews(false);
      callback(EMPTY);
      return;
    }

    markTopicHasOlderNews(true);

    mCurrentDetailTopic.appendSize = 0;

    var requestedNewsIDs = [];
    for (var i = 0, max = data.length; i < max; i++) {
      var news = data[i];
      requestedNewsIDs.push(news.id);
      insertOrUpdateCategoryNews(news);
      updateTopicTimemark(news.publish_time);
    }

    if (isShowComments()) {
      getNewsInteractionsCounterBatch(requestedNewsIDs, function() {
        onEvent(EVENT_ELEMENT_REFRESH);
      });
    }

    mergeTopicNewsEntrance();

    mCurrentDetailTopic.newsArray.length = 0;
    mCurrentDetailTopic.newsArray = getTopicNewsArray(topicID, mCategoryNews);

    //storeCategoryNews();

    callback(SUCCESS);
  }, function(data, statusCode) {
    callback(FAILURE);
  });
}

/**
 * 获取 较旧的 专题新闻
 * @param categoryID
 * @param topicID
 * @param requestSize
 * @param callback
 */
function getTopicNewsOlder(topicID, requestSize, callback) {
  var size = 0;
  if (mCurrentDetailTopic !== undefined) {
    $.getJSON("/news", {
      "topic": topicID,
      "mints": mCurrentDetailTopic.minTs,
      "size": requestSize
    }, function(data) {
      data = data.items;

      if (data) {
        size = data.length;
      }
      markTopicNewsRequested();

      if (data === undefined || data.length <= 0) {
        markTopicHasOlderNews(false);
        callback(EMPTY, size);
        return;
      }

      markTopicHasOlderNews(true);

      var requestedNewsIDs = [];
      for (var i = 0, max = data.length; i < max; i++) {
        var news = data[i];
        requestedNewsIDs.push(news.id);

        insertOrUpdateCategoryNews(news);
        updateTopicTimemark(news.publish_time);
      }

      if (isShowComments()) {
        getNewsInteractionsCounterBatch(requestedNewsIDs, function() {
          onEvent(EVENT_ELEMENT_REFRESH);
        });
      }

      mergeTopicNewsEntrance();

      mCurrentDetailTopic.newsArray.length = 0;
      mCurrentDetailTopic.newsArray = getTopicNewsArray(topicID, mCategoryNews);

      callback(SUCCESS, size);
    }, function(data, statusCode) {
      markTopicHasOlderNews(false);
      callback(FAILURE, size);
    });
  } else {
    callback(FAILURE);
  }
}

/**
 * 获取现有 所有专题ID
 * @returns {Array}
 */
function getAllRemainTopicIds() {
  var allTopicIds = [];
  for (var i = 0, max = mCategoryNews.length; i < max; i++) {
    var news = mCategoryNews[i];
    if (news !== undefined && isTypeOfTopic(news)) {
      insertArrayToArray(allTopicIds, news.topics);
    }
  }
  return allTopicIds;
}

/**
 * 合并更新 专题入口
 */
function mergeTopicNewsEntrance() {
  var topicIds = getAllRemainTopicIds();
  for (var i = 0, tmax = topicIds.length; i < tmax; i++) {
    var topicId = topicIds[i];
    var topicNews = getTopicNewsArray(topicId, mCategoryNews);
    sortNewsByPublishTime(topicNews);
    var foundEntry = false;
    for (var j = 0, tnmax = topicNews.length; j < tnmax; j++) {
      if (foundEntry) {
        topicNews[j].topic_flag = -1;
      } else {
        foundEntry = topicNews[j].topic_flag === 0;
      }
    }
  }
}

// 用户信息缓存
var mUserInfoCache = [];

function buildUserInfo(userInfo) {
  var info = new UserInfo(userInfo.uid);
  updateUserInfo(userInfo, info);
  return info;
}

function getIpArea(userinfo) {
  userinfo = JSON.parse(userinfo);
  var info = userinfo.org_exinfo && userinfo.org_exinfo.ip_area;

  if (!userinfo.org_exinfo || !info || info.length === 0) {
    return [window.translate('unknown'), window.translate('unknown')];
  }

  info = info.match(/[\u4E00-\u9FA5\w\s]+/g);

  if (info) {
    switch (info.length) {
      case 2:
        return [info[1], window.translate('unknown')];
      case 3:
      case 4:
        return [info[1], info[2]];
      default:
        return [window.translate('unknown'), window.translate('unknown')];
    }
  }
}

function updateUserInfo(src, dest) {
  dest.name = src.name;
  // dest.user_name = src.user_name;
  // dest.nick_name = src.nick_name;
  // dest.real_name = src.org_exinfo.real_name;
  // dest.nick_name_full = src.nick_name_full;
  // dest.nick_name_short = src.nick_name_short;
}

function findUserInfo(uid) {
  return findItem(mUserInfoCache, uid);
}

/**
 * 插入或更新 用户信息
 * @param userInfo
 */
function insertOrUpdateUserInfo(userInfo) {
  if (!userInfo || !userInfo.uid) return;

  var nowTime = getNowUnixTime();
  var found = findUserInfo(userInfo.uid);

  if (!found) {
    var info = buildUserInfo(userInfo);
    info.last_update_time = nowTime;
    mUserInfoCache.push(info);
  } else {
    updateUserInfo(userInfo, found);
    found.last_update_time = nowTime;
  }

}

/**
 * 判断用户信息 是否已经过期
 * @param uid
 * @returns {boolean}
 */
function isUserInfoOutOfDate(uid) {
  var userInfo = findUserInfo(uid);
  if (userInfo === undefined) return true; // 找不到用户信息，返回 true，表示需要更新
  // 用户信息过期时间间隔：30分钟
  return getNowUnixTime() - userInfo.last_update_time > (1000 * 60 * 30);
}

/**
 * 读取用户名称
 * @param uid
 * @returns {*}
 */
function getUserNameFromCache(uid) {
  if (!uid) {
    return window.translate("anonymous");
  }
  var userInfo = findUserInfo(uid);
  if (userInfo) {
    return userInfo.name || userInfo.id;
  }

  return uid;
}

/**
 * 批量更新用户信息
 * @param uids
 * @param callback
 */
function updateCommentUserNameBatch(uids, callback) {
  if (!uids || !uids.length) {
    callback(FAILURE);
    return;
  }

  uids = uids.filter(function(uid) {
    return !!uid;
  });

  if (!uids.length) {
    callback(FAILURE);
    return;
  }

  getUsersName(uids, function(data) {
    if (!data || !data.items) {
      callback(FAILURE);
      return;
    }

    data.items.forEach(function(item) {
      insertOrUpdateUserInfo(item);
    });

    callback(SUCCESS);
  }, function(err) {
    callback(FAILURE);
  });
}

if (mPlatformIOS) {
  (function() {
    function listener() {
      MUICordovaWebJSExport.allowURLJump();
      document.removeEventListener('touchstart', listener, false);
    }

    document.addEventListener('touchstart', listener, false);
  })();
}

var TIME_OPTIONS = {
  day: 24 * 60 * 60 * 1000,
  minute: 60 * 60 * 1000,
  second: 60 * 1000
};

/**
 * 发布时间格式化处理
 */
function timeFormat(news) {
  var now = new Date().getTime();
  var time = news.publish_time;
  var diff = now - time;
  var newTime;

  if (diff > TIME_OPTIONS.day) {
    newTime = Math.floor(diff / (TIME_OPTIONS.day)) + window.translate('news_list_days_ago');
  } else if (diff > TIME_OPTIONS.minute) {
    newTime = Math.floor(diff / (TIME_OPTIONS.minute)) + window.translate('news_list_hours_ago');
  } else if (diff > TIME_OPTIONS.second) {
    newTime = Math.floor(diff / (TIME_OPTIONS.second)) + window.translate('news_list_minutes_ago');
  } else {
    newTime = window.translate('news_list_just_now');
  }

  return newTime;
}

/**
 * 是否显示评论
 */
function isShowComments() {
  return COMMENTABLE === 'true' ? true : false;
}

function isShowShare() {
  return SOCIALSHARE === 'true' ? true : false;
}

/**
 * 判断是否是游客模式
 */
function isGuest() {
  if (mPlatformAndroid) {
    return JsHttpRequest.isGuest();
  } else if (mPlatformIOS) {
    return MUICordovaWebJSExport.isGuest();
  }
}

/**
 * 游客模式，获取用户名的url
 */
function getGuestUserInfoUrl() {
  if (mPlatformAndroid) {
    return JsHttpRequest.getGuestUserInfoUrl();
  } else if (mPlatformIOS) {
    return MUICordovaWebJSExport.getGuestUserInfoUrl();
  }
}

/**
 * 从 native 批量获取用户信息
 */
function getUsersName(ids, success, failure) {
  var data = JSON.stringify({
    uids: ids
  });

  var str = new Date().getTime().toString(16) + Math.random().toString(16).substring(2);
  var scb = 's' + str;
  var fcb = 'f' + str;

  window[scb] = success;
  window[fcb] = failure;

  if (mPlatformAndroid) {
    JsHttpRequest.getUsersName(data, scb, fcb);
  } else if (mPlatformIOS) {
    MUICordovaWebJSExport.getUsersName(data, scb, fcb);
  }
}

/**
 * 举报评论
 */
function report(comment) {
  hideInputView();

  $('.mask').css('height', $(document).height()).show();
  $('.report-container').attr('data-id', comment.id).slideUp('slow').show();
}

function hideInputView() {
  if (mPlatformAndroid) {
    CommentService.hideInputView();
  } else if (mPlatformIOS) {
    MUICordovaWebJSExport.hideInputView();
  }
}

function showInputView() {
  if (mPlatformAndroid) {
    CommentService.showInputView();
  } else if (mPlatformIOS) {
    MUICordovaWebJSExport.showInputView();
  }
}

/**
 * 取消 举报
 */
function cancelReport() {
  $('.mask, .report-container').hide();

  showInputView();
}

function confirmReport(type, text) {
  var cmt_id = $('.report-container').attr('data-id');
  var data = {
    'biz_type': 'NEWS',
    'cmt_id': cmt_id,
    'type': type,
    'desc': text
  };

  $.postJSON(HOST_SDP_INTERACTIONS + 'reports', data, function(data) {
    cancelReport();
    Toast(window.translate('report_success'));
  }, function(error) {
    cancelReport();
  });

}

/**
 * 举报评论
 */
function deleteCmt(comment) {
  hideInputView();

  $('.mask').css('height', $(document).height()).show();
  $('.delete-container').attr('data-id', comment.id).slideUp('slow').show();
}

/**
 * 取消 举报
 */
function cancelDelete() {
  $('.mask, .delete-container').hide();

  showInputView();
}

function confirmDelete(callback) {
  var cmt_id = $('.delete-container').attr('data-id');

  $.deleteJSON(HOST_SDP_INTERACTIONS + 'comments/' + cmt_id, {biz_type:'NEWS'}, function(data) {
    mCurrentComments.map(function(item, i) {
      if (item.id === cmt_id) {
        mCurrentComments.splice(i, 1);
      }
    });

    window.onPullDownToRefresh();
    cancelDelete();
    Toast(window.translate('delete_success'));
    if (callback) {
      callback(data);
    }
  }, function(error) {
    // Toast(window.translate('delete_fail'));
  });

}

/**
 * 文本溢出省略号
 */
function clamp() {
  var title = $('.list-news-title');
  title.map(function(i, item) {
    $clamp(item, {
      clamp: 2
    });
  });
}

/**
 * 图片轮播
 */
var swiper;

function slide() {
  //页面没有加载好，延迟
  if (!$('.detail-error-text').length && !$('.news-list').length) {
    return setTimeout(function() {
      slide();
    }, 100);
  }

  var slideBox = $('#slideBox');
  var oUl = $('#slideBox ul');
  var oli = $('#slideBox li');
  var len = oli.length;
  var width = len * oli.width();
  var height = oli.height();
  var bullet_width = oli.width() / len;

  if (len > 0) {
    if (swiper) {
      swiper.destroy(true, true);
    }

    slideBox.css({
      'height': height,
      'display': 'block'
    });

    oUl.css({
      'width': width,
      'height': height
    });

    swiper = new Swiper('#slideBox', {
      pagination: '.swiper-pagination',
      paginationBulletRender: function(index, className) {
        return '<span class="' + className + '"><span class="current">' + (index + 1) + '</span>/<span>' + len + '</span></span>';
      },
      /*paginationBulletRender: function(index, className) {
        return '<span class="' + className + '" style="width: ' + bullet_width + 'px;"></span>';
      },*/
      onTouchStart: function() {
        if (mPlatformAndroid) {
          webView.isChildSliding(true);
        }
      },
      onTouchEnd: function() {
        if (mPlatformAndroid) {
          webView.isChildSliding(false);
        }
      }
    });

    if (mPlatformAndroid) {
      swiper.slideNext();
      swiper.slidePrev();
    }

  } else {
    slideBox.css('display', 'none');
  }

}

/**
 * 获取用户id
 */
function getCurrentUid() {
  if (mPlatformAndroid) {
    return JsHttpRequest.getCurrentUid();
  }
  if (mPlatformIOS) {
    return MUICordovaWebJSExport.getUserUid();
  }
}

/**
 * 友盟统计
 */
function analyticsEvent(eventId, param) {
  if (mPlatformAndroid) {
    AnlysisTools.onEvent(eventId, param);
  }
  if (mPlatformIOS) {
    MUICordovaWebJSExport.dataAnalyticsEvent(eventId, param);
  }
}

/**
 * 获取用户信息
 */
function getLocaleUserInfo() {
  if (mPlatformAndroid) {
    return JsSaveSp.getItem('user_info');
  }
  if (mPlatformIOS) {
    return localStorage.getItem('user_info');
  }
}

var a = 'getCurrentUserSuccess';
var b = 'getCurrentUserFail';

//获取用户信息的成功回调
window[a] = function(data) {
  if (mPlatformAndroid) {
    if (!JsSaveSp.getItem('user_info')) {
      JsSaveSp.removeItem('user_info');
      JsSaveSp.setItem('user_info', JSON.stringify(data));
    }
  }
  if (mPlatformIOS) {
    localStorage.removeItem('user_info');
    localStorage.setItem('user_info', JSON.stringify(data));
  }
};
//获取用户信息的失败回调
window[b] = function(data) {
  console.log("get user info error");
};

if (mPlatformAndroid) {
  JsHttpRequest.getCurrentUser(a, b);
}
if (mPlatformIOS) {
  MUICordovaWebJSExport.getUserInfo(a, b);
}
