angular.module('news.service.category', [])
  .service('categoryService', function() {

    this.buildCategory = function(data) {
      var category = new Category(data.id);
      updateCategory(data, category);
      return category;
    };

    this.updateCategory = function(src, dest) {
      if (src === undefined || dest === undefined) return;
      dest.desc = src.desc;
      dest.name = src.name;
      dest.sort = src.sort;
    };

    /**
     * 插入或更新 新闻的分类
     * @param category
     * @returns {*}
     */
    this.insertOrUpdateCategory = function(categories, category) {

      if (category.id === undefined) return;

      var found = findCategory(category.id);
      if (found === undefined) {
        categories.push(buildCategory(category));
      } else {
        updateCategory(category, found);
      }

    };

    this.findCategory = function(categories, categoryID) {
      return findItem(categories, categoryID);
    };


    /*******************************************************************************/

    /**
     * 判断 分类新闻是否获取过
     * @param categoryID
     * @returns {boolean}
     */
    this.isCategoryNewsRequested = function(category) {
      if (category !== undefined) {
        return category.newsRequested;
      }
      return false;
    };

    /**
     * 标记 分类新闻获取过
     * @param categoryID
     */
    this.markCategoryNewsRequested = function(category) {
      if (category !== undefined) {
        category.lastUpdateNewsTime = Math.max(category.maxTs, getNowUnixTime());
        category.newsRequested = true;
      }
    };

    /**
     * 判断 分类对应的新闻是否为空
     * @param categoryID
     * @returns {*}
     */
    this.isCategoryHasNews = function(category) {
      if (category !== undefined) {
        return category.hasNews;
      }
      return false;
    };

    /**
     * 标记 分类对应的新闻不为空
     * @param categoryID
     */
    this.markCategoryHasNews = function(category) {
      if (category !== undefined) {
        category.hasNews = true;
      }
    };

    /**
     * 自动检测 分类对应的新闻是否为空
     * @param categoryID
     */
    this.checkCategoryHasNews = function(categories, categoryID) {
      if (categoryID === undefined) return;

      var category = findCategory(categories, categoryID);
      if (category !== undefined) {
        category.hasNews = category.newsArray.length > 0;
      }
    };


    /**
     * 判断 分类对应的新闻是否还有早期新闻需要获取
     * @param categoryID
     * @returns {*}
     */
    this.isCategoryHasOlderNews = function(category) {
      if (category !== undefined) {
        return category.moreOlderNews && category.minTs != INVALID;
      }
      return false;
    };

    /**
     * 标注 分类对应的新闻是否还有早期新闻需要获取
     * @param categoryID
     * @param hasOlderNews
     * @returns {*}
     */
    this.markCategoryHasOlderNews = function(category, hasOlderNews) {
      if (category !== undefined) {
        category.moreOlderNews = hasOlderNews;
      }
    };

    /**
     * 刷新 分类新闻的更新时间戳
     * @param categoryID
     * @param time
     */
    this.updateCategoryTimemark = function(category, time) {
      if (category !== undefined) {
        // min
        if (INVALID === category.minTs || time < category.minTs) {
          category.minTs = time;
        }

        // max
        if (INVALID === category.maxTs || time > category.maxTs) {
          category.maxTs = time;
        }
      }
    };

    /**
     * 获取 分类上一次请求新闻的时间
     * @param categoryID
     * @returns {*}
     */
    this.getLastUpdateNewsTime = function(category) {
      if (category !== undefined && category.lastUpdateNewsTime > 0) {
        return new Date(category.lastUpdateNewsTime).format(DEFAULT_TIME_FORMAT);
      }
      return "暂无记录";
    };

    /*******************************************************************************/

    /**
     * 获取 新闻的分类
     * @param callback
     */
    this.getCategories = function(categories, callback) {
      $.getJSON("/news/categories", {}, function(data) {
        log("getCategories :: fromServer :: " + data.items.length);

        if (data === undefined || data.length <= 0) {
          callback(data, EMPTY);
          return;
        }

        for (var j = 0; j < data.items.length; j++) {
          var item = data.items[j];
          insertOrUpdateCategory(categories, item);
        }

        //storeCategories();

        callback(data, SUCCESS);

      }, function(data, statusCode) {
        log("getCategories :: error");
        log(data);
        callback(data, FAILURE);
      });
    };

  });
