angular.module('news.service.topic', ['common', 'newsService'])
  .service('topicService', function($common, $newsService) {

    this.buildTopic = function(data) {
      if (data === undefined) return undefined;
      var topic = new Topic(data.id);
      topic.create_time = data.create_time;
      topic.name = data.name;
      topic.summary = data.summary;
      topic.update_time = data.update_time;
      topic.cover_img = data.cover_img;
      return topic;
    };

    var _updateTopic = function(topic, data) {
      if (topic === undefined || data === undefined) return;
      topic.create_time = data.create_time;
      topic.name = data.name;
      topic.summary = data.summary;
      topic.update_time = data.update_time;
      topic.cover_img = data.cover_img;
    };


    var _insertOrUpdateTopicNews = function(topic, news) {
      if (topic === undefined || news === undefined) return false;

      var found = $common.findItem(topic.newsArray, news.id);
      var needToInsert = found === undefined;
      if (needToInsert) {
        topic.newsArray.push($newsService.buildNews(news));
      } else {
        $newsService.updateNewsBase(news, found);
      }

      return needToInsert;
    };

    /**
     * 获取 专题新闻 上次请求的时间
     * @returns {*}
     */
    this.getTopicNewsLastUpdateTime = function(topic) {
      if (topic !== undefined && topic.lastUpdateNewsTime > 0) {
        return new Date(topic.lastUpdateNewsTime).format(DEFAULT_TIME_FORMAT);
      }
      return "暂无记录";
    };

    /**
     * 标注 专题新闻 是否被获取过
     */
    var _markTopicNewsRequested = function(topic) {
      if (topic !== undefined) {
        topic.lastUpdateNewsTime = Math.max(topic.maxTs, getNowUnixTime());
        topic.newsRequested = true;
      }
    };

    /**
     * 标注 专题新闻 是否有较旧的新闻需要获取
     * @param has
     */
    var _markTopicHasOlderNews = function(topic, has) {
      if (topic !== undefined) {
        topic.moreOlderNews = has;
      }
    };

    /**
     * 判断 当前专题 是否有 较早的新闻需要加载
     * @returns {*}
     */
    this.isTopicHasOlderNews = function(topic) {
      if (topic === undefined) return false;
      return topic.moreOlderNews && topic.minTs != INVALID;
    };

    /**
     * 判断 当前专题 是否有 新闻
     * @returns {boolean}
     */
    this.isTopicHasNews = function(topic) {
      return topic.newsArray.length > 0;
    };

    /**
     * 获取 专题详情
     * */
    this.getTopicDetail = function(topic, requestSize, callbackTopic, callbackNews) {
      $.getJSON("/news/topics/" + topic.id, {}, function(data) {
        log("getTopicDetail :: topicID :: " + topic.id);

        _updateTopic(topic, data);

        callbackTopic(SUCCESS);

        // 获取最新的专题新闻
        this.getTopicNewsLatest(topic.id, requestSize, callbackNews);

      }, function(data, statusCode) {
        log("getTopicDetail :: error :: " + topicID);
        log(data);
        callbackTopic(FAILURE);
      });
    };

    /**
     * 判断 新闻是否属于 分类
     * @param news
     * @param categoryID
     * @returns {boolean}
     */
    this.isNewsBelongTopic = function(news, topicID) {
      if (news === undefined) return false;

      var topics = news.topics;
      for (var i = 0; i < topics.length; i++) {
        if (topics[i] === topicID) {
          return true;
        }
      }
      return false;
    };

    /**
     * 获取 属于专题的新闻 数组对象
     * @param topicID
     * @param allNews
     * @returns {Array}
     */
    this.getTopicNewsArray = function(topicID, allNews) {
      log("getTopicNewsArray :: " + topicID + " , " + allNews.length);

      var result = [];
      if (allNews != undefined && allNews.length > 0) {
        for (var i = 0; i < allNews.length; i++) {
          var news = allNews[i];
          if (isNewsBelongTopic(news, topicID)) {
            result.push(news);
          }
        }
      }
      return result;
    };

    /**
     * 刷新 分类新闻的更新时间戳
     * @param categoryID
     * @param time
     */
    var _updateTopicTimemark = function(topic, time) {
      if (topic !== undefined) {
        // min
        if (INVALID === topic.minTs || time < topic.minTs) {
          topic.minTs = time;
        }

        // max
        if (INVALID === topic.maxTs || time > topic.maxTs) {
          topic.maxTs = time;
        }
      }
    };

    /**
     * 获取 最新的 专题新闻
     * @param categoryID
     * @param topicID
     * @param requestSize
     * @param callback
     */
    this.getTopicNewsLatest = function(topic, requestSize, callback) {
      if (topic === undefined) {
        callback(FAILURE);
        return;
      }

      var maxTs = (topic !== undefined && topic.maxTs != INVALID) ? topic.maxTs : 0;
      log("getTopicNewsLatest :: topicID-->" + topic.id + " , maxTs-->" + maxTs + " , requestSize-->" + requestSize);
      $.getJSON("/news", {
        "topic": topic.id,
        "maxts": maxTs,
        "start": 0,
        "size": requestSize
      }, function(data) {

        _markTopicNewsRequested(topic);

        if (data === undefined || data.length <= 0) {
          _markTopicHasOlderNews(topic, false);
          callback(EMPTY);
          return;
        }

        _markTopicHasOlderNews(true);

        topic.appendSize = 0;

        var requestedNewsIDs = [];
        for (var i = 0; i < data.length; i++) {
          var news = data[i];
          requestedNewsIDs.push(news.id);

          if (_insertOrUpdateTopicNews(topic, news)) {
            topic.appendSize++;
          }

          _updateTopicTimemark(news.publish_time);
        }

        //getNewsInteractionsCounterBatch( requestedNewsIDs, function(){} );

        //storeCategoryNews();

        callback(SUCCESS);
      }, function(data, statusCode) {
        log("getTopicNewsLatest :: error");
        log(data);
        callback(FAILURE);
      });
    };

    /**
     * 获取 较旧的 专题新闻
     * @param categoryID
     * @param topicID
     * @param requestSize
     * @param callback
     */
    function getTopicNewsOlder(topicID, requestSize, callback) {

      if (mCurrentDetailTopic !== undefined) {
        $.getJSON("/news", {
          "topic": topicID,
          "mints": mCurrentDetailTopic.minTs,
          "size": requestSize
        }, function(data) {
          log("getTopicNewsOlder");
          log(data);

          markTopicNewsRequested();

          if (data === undefined || data.length <= 0) {
            markTopicHasOlderNews(false);
            callback(EMPTY);
            return;
          }

          markTopicHasOlderNews(true);

          var requestedNewsIDs = [];
          for (var i = 0; i < data.length; i++) {
            var news = data[i];
            requestedNewsIDs.push(news.id);

            insertOrUpdateCategoryNews(news);
            updateTopicTimemark(news.publish_time);
          }

          getNewsInteractionsCounterBatch(requestedNewsIDs, function() {});

          callback(SUCCESS);
        }, function(data, statusCode) {
          markTopicHasOlderNews(false);
          callback(FAILURE);
        });
      } else {
        callback(FAILURE);
      }
    }

  });
